# Generated by Django 4.2.16 on 2026-01-18 08:55

from django.db import migrations

from ..tasks.updates import clear_all_skill_caches


def clear_cache(apps, schema_editor):
    print(clear_all_skill_caches())


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0127_alter_corporationaudit_options"),
    ]

    operations = [
        migrations.RunPython(clear_cache, clear_cache)
    ]



import pprint

from ninja import NinjaAPI

from django.core.cache import cache
from django.utils import timezone
from django.utils.http import http_date

from esi.exceptions import HTTPNotModified

from corptools.task_helpers.char_tasks import get_token


class TestingApiEndpoints:
    tags = ["Extras"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "/extras/test/newapi1",
            tags=["Testing"]
        )
        def get_test_api_bravadosih(request):
            """
                Not for real use!
                this is kinda like bravado in use... no type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from esi.clients import OpenAPIEsiClientProvider
            api = OpenAPIEsiClientProvider()

            return api.client.Alliance.get_alliances_alliance_id(alliance_id=1900696668)

        @api.get(
            "/extras/test/skilltest",
            tags=["Testing"]
        )
        def get_test_api_skilltests(request):
            """
                Not for real use!
                this is kinda like bravado in use... no type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from corptools.providers import skills
            skills.get_and_cache_user(request.user.id, force_rebuild=True)

            return skills.get_and_cache_user(request.user.id)

        @api.get(
            "/extras/test/newapi2",
            tags=["Testing"]
        )
        def get_test_api_openapi(request):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from esi.clients import OpenAPIEsiClientProvider
            api = OpenAPIEsiClientProvider()

            from esi.client.api import AllianceApi

            return AllianceApi(api.client).get_alliances_alliance_id(alliance_id=1900696668)

        @api.get(
            "/extras/test/newapi3",
            tags=["Testing"]
        )
        def get_test_api_openapi_3(request, name=""):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from corptools.models import EveItemType
            item, created = EveItemType.objects.get_or_create_from_esi_name(
                name)
            return {
                "created": created,
                "item_name": item.name,
                "item_id": item.type_id
            }

        @api.get(
            "/extras/test/newapi4",
            tags=["Testing"]
        )
        def get_test_api_openapi_4(request, character_id=0):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from corptools.task_helpers.char_tasks import (
                update_character_assets,
            )

            update_character_assets(character_id, force_refresh=True)
            return "done"

        @api.get(
            "/extras/test/newapi5",
            tags=["Testing"]
        )
        def get_test_api_oiaopenapi3(request):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}
            # from aiopenapi3 import OpenAPI

            # from esi.clients import EsiClientProvider
            # esi2 = EsiClientProvider()
            # res = esi2.client.Sovereignty.get_sovereignty_campaigns()
            # res.request_config.also_return_response = True
            # result, headers = res.result()
            # etag1 = headers.headers["etag"]
            # print("here")
            # res = esi.client.Alliance.GetAlliances(
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # res = esi.client.Alliance.GetAlliancesAllianceId(
            #     alliance_id=1900696668
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # res = esi.client.Status.GetStatus(
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # res = esi.client.Incursions.GetIncursions(
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # from esi.models import Token
            # from esi.rate_limiting import ESIRateLimitBucket
            # from corptools.models import CharacterAsset, MapSystem
            # esi = openapi_clients.ESIClientProvider(
            #     compatibility_date="2025-08-26",
            #     ua_appname="ack-testing-django-esi",
            #     ua_version="0.0.1a1"
            # )
            # def hit_api():
            #     esi.client.Alliance.GetAlliances(
            # ).result()
            # for i in range(0,50):
            #     print(f"Hit {i}")
            #     hit_api()
            # req_scopes = ['esi-assets.read_assets.v1']
            # req_scopes = ['esi-fleets.write_fleet.v1']
            # cid = 2119950231
            # cid = 755166922
            # token = get_token(cid, req_scopes)
            # try:
            #     esi.client.Alliance.GetAlliances(
            #         token=token
            #     ).result()
            # except ValueError:
            #     print("This is good")
            # res = esi.client.Assets.GetCharactersCharacterIdAssets(
            #     character_id=755166922,
            #     token=token
            # )
            # result, headers = res.results(return_response=True)
            # print(result[0])
            # for a in result:
            #     print(a.type_id)
            # flt = esi.client.Fleets.GetCharactersCharacterIdFleet(
            #     body={"is_free_move": True, "motd": "Testing 1, 2, 3"},
            #     character_id=cid,
            #     token=token
            # ).result()
            # flt_up = esi.client.Fleets.PutFleetsFleetId(
            #     body={"is_free_move": True, "motd": "Testing 1, 2, 3"},
            #     fleet_id=1234,
            #     token=token
            # )
            # flt_up.result()
            # print(flt)
            # print(flt_up)
            # res = esi.client.Assets.GetCharactersCharacterIdAssets(
            #     character_id=755166922,
            #     token=Token.objects.all().require_scopes_exact("publicData").first()
            # )
            # result, headers = res.results(return_response=True)
            # print([len(result), headers])
            # from esi import clients as clients2
            # esi2 = clients2.EsiClientProvider(
            #     ua_appname = "ack-testing-django-esi",
            #     ua_version = "0.0.1a1"
            # )
            # token = get_token(755166922, req_scopes)
            # res = esi2.client.Assets.get_characters_character_id_assets(
            #     character_id=755166922,
            #     token=token.valid_access_token()
            # )
            # print(etag1)
            # from esi import openapi_clients
            # esi = openapi_clients.ESIClientProvider(
            #     compatibility_date="2025-08-26",
            #     ua_appname="ack-testing-django-esi",
            #     ua_version="0.0.1a1",
            #     tags=["Character", "Assets", "Skills"]
            # )
            # # try:
            # #     req, res = esi.client.Universe.GetUniverseCategoriesCategoryId(category_id=999999).result(use_etag=False, return_response=True)
            # # except Exception as e:
            # #     print(e)

            # # req, res = esi.client.Universe.GetUniverseCategoriesCategoryId(category_id=5).result(return_response=True)
            # # print(res.headers)
            # from django.core.cache import cache

            # req_scopes = ['esi-skills.read_skills.v1']
            # cid = 755166922
            # token = get_token(cid, req_scopes)
            # op = esi.client.Skills.GetCharactersCharacterIdSkills(character_id=cid, token=token)
            # etag_key = op._etag_key()
            # print("TTL_pre", cache.ttl(etag_key))
            # try:
            #     op.result(use_cache=False)
            # except Exception:
            #     pass
            # print("TTL_post", cache.ttl(etag_key))
            # req = esi.client.Character.GetCharactersCharacterIdNotifications(character_id=cid, token=token).result()

            # req_scopes = ['esi-characters.read_corporation_roles.v1']
            # token = get_token(cid, req_scopes)
            # now = timezone.now()
            # req = esi.client.Character.GetCharactersCharacterIdRoles(
            #     character_id=cid,
            #     token=token
            # ).result(
            #     use_cache=False,
            #     use_etag=False,
            #     extra_headers={
            #         "If-Modified-Since": http_date(now.timestamp())
            #     }
            # )

            # try:
            #     req, res = esi.client.Assets.GetCharactersCharacterIdAssets(
            #         character_id=cid, token=token).results(return_response=True)
            # except HTTPNotModified:
            #     pass
            # cache.clear()
            # req_scopes = ['esi-characters.read_notifications.v1']
            # token = get_token(cid, req_scopes)
            # req, res = esi.client.Character.GetCharactersCharacterIdNotifications(
            #     character_id=cid, token=token).results(return_response=True, use_cache=False, use_etag=False)
            # return [req]

            from django.core.cache import cache

            from corptools.providers import esi_openapi
            from corptools.task_helpers.char_tasks import (
                update_character_skill_list,
            )

            def test_():
                cid = 2117840319
                op = esi_openapi.client.Skills.GetCharactersCharacterIdSkills(
                    character_id=cid)
                etag_key = op._etag_key()
                print("TTL_pre", cache.ttl(etag_key))
                try:
                    update_character_skill_list(cid, force_refresh=False)
                except Exception:
                    pass
                print("TTL_post", cache.ttl(etag_key))
            test_()



from django.core.management.base import BaseCommand


class ACLGroup:
    """
        Basically a set split into alli/corp/character for sorting in ACLs
    """
    prefix = ""
    alliance: set
    corporation: set
    character: set

    def __init__(self, prefix: str = "") -> None:
        self.prefix = prefix
        self.alliance = set()
        self.corporation = set()
        self.character = set()

    def add(self, name, cat="character"):
        if cat == "character":
            self.character.add(name)
        if cat == "corporation":
            self.corporation.add(name)
        if cat == "alliance":
            self.alliance.add(name)

    def remove(self, name):
        if name in self.alliance:
            self.alliance.remove(name)
        if name in self.corporation:
            self.corporation.remove(name)
        if name in self.character:
            self.character.remove(name)

    def __contains__(self, key):
        return key in self.alliance or key in self.corporation or key in self.character

    def __len__(self):
        return len(self.alliance) + len(self.corporation) + len(self.character)

    def __iter__(self):
        acl_list = []
        acl_list += sorted(list(self.alliance))
        acl_list += sorted(list(self.corporation))
        acl_list += sorted(list(self.character))
        return iter(acl_list)

    def __str__(self) -> str:
        return f"{self.name}"


class ACL:
    name = "An ACL"
    members: ACLGroup
    managers: ACLGroup
    admins: ACLGroup

    def __init__(self, name: str) -> None:
        self.name = name
        self.members = ACLGroup()
        self.managers = ACLGroup()
        self.admins = ACLGroup()

    def add_to_group(self, group, name, cat="character"):
        if group == "member":
            self.members.add(name, cat=cat)
        elif group == "manager":
            self.managers.add(name, cat=cat)
        elif group == "admin":
            self.admins.add(name, cat=cat)

    def rem_from_group(self, group, name, new_group=None, cat="character"):
        if group == "member":
            self.members.remove(name)
        elif group == "manager":
            self.managers.remove(name)
        elif group == "admin":
            self.admins.remove(name)

        if new_group:
            self.add_to_group(new_group, name, cat=cat)

    def __repr__(self):
        return str(
            {
                "Admin": list(self.admins),
                "Managers": list(self.managers),
                "Members": list(self.members)
            }
        )


class Command(BaseCommand):
    help = 'Check '

    def handle(self, *args, **options):
        import re

        regex = r"^([0-9\.]{10})\s{1}([0-9\:]{8})\s{1}(.*)\s{1}(added|removed|remove|changed)\s{1}([\w\W ]{1,32})( as | to | \()(member|manager|blocked|admin)( from )?(member|manager|blocked|admin)?"

        with open('testacl.txt', 'r') as file:
            lines = file.readlines()
            # Join the lines
            content = '\n'.join(reversed(lines))

            matches = re.finditer(regex, content, re.MULTILINE)

        acl = ACL("ACL Test")
        names = []
        lookup = set()

        for matchNum, match in enumerate(matches, start=1):
            act = match.group(4)
            grp = match.group(7)
            nme = match.group(5)
            cng = match.group(9)
            names.append([act, grp, nme, cng])
            lookup.add(nme)

        from corptools.providers import esi
        lookup = esi.client.Universe.post_universe_ids(
            names=list(lookup)).results()

        corps = [i["name"]
                 for i in (lookup["corporations"] if lookup["corporations"] else [])]
        allis = [i["name"]
                 for i in (lookup["alliances"] if lookup["alliances"] else [])]
        for name in names:
            cat = "character"
            if name[2] in corps:
                cat = "corporation"
            elif name[2] in allis:
                cat = "alliance"
            if name[0] == "added":
                acl.add_to_group(name[1], name[2], cat=cat)
            elif name[0] in ["removed", "remove"]:
                acl.rem_from_group(name[1], name[2])
            elif name[0] in ["changed"]:
                acl.rem_from_group(
                    name[3], name[2], new_group=name[1], cat=cat)

        print(acl)
        print("ACL Should have ", len(acl.admins), "Admins")
        entities = sorted(list(acl.admins))
        print(list(entities))
        print("ACL Should have ", len(acl.managers), "Managers")
        entities = sorted(list(acl.managers))
        print(list(entities))
        print("ACL Should have ", len(acl.members), "Members")
        entities = sorted(list(acl.members))
        print(list(entities))



# Generated by Django 4.0.2 on 2022-05-16 10:39

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0073_remove_miningtaxpaymentcorp_corp_delete_miningtax_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='evelocation',
            name='managed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='evelocation',
            name='managed_char',
            field=models.ForeignKey(blank=True, default=None, null=True,
                                    on_delete=django.db.models.deletion.CASCADE, to='corptools.characteraudit'),
        ),
        migrations.AddField(
            model_name='evelocation',
            name='managed_corp',
            field=models.ForeignKey(blank=True, default=None, null=True,
                                    on_delete=django.db.models.deletion.CASCADE, to='corptools.corporationaudit'),
        ),
    ]



import json

from jsonschema import ValidationError

from django.db import models
from django.utils.translation import gettext_lazy as _

from .audits import CharacterAudit
from .eve_models import EveItemType


class SkillTotals(models.Model):
    character = models.OneToOneField(CharacterAudit, on_delete=models.CASCADE)

    total_sp = models.BigIntegerField()
    unallocated_sp = models.IntegerField(null=True, default=None)


class SkillTotalHistory(models.Model):
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now=True)
    total_sp = models.BigIntegerField()
    unallocated_sp = models.IntegerField(default=0)

    @property
    def sp(self):
        return self.total_sp + self.unallocated_sp


class Skill(models.Model):
    id = models.BigAutoField(primary_key=True)
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)
    skill_id = models.IntegerField()
    skill_name = models.ForeignKey(
        EveItemType, on_delete=models.CASCADE, null=True, default=None)
    active_skill_level = models.IntegerField()
    skillpoints_in_skill = models.BigIntegerField()
    trained_skill_level = models.IntegerField()

    @property
    def alpha(self):
        if self.trained_skill_level == self.active_skill_level:
            return False
        return True  # is alpha clone

    class Meta:
        unique_together = (("character", "skill_id"),)

# Skill Queue Model


class SkillQueue(models.Model):
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)
    # Required Fields / Fields Always Present
    finish_level = models.IntegerField()
    queue_position = models.IntegerField()
    skill_id = models.IntegerField()
    skill_name = models.ForeignKey(
        EveItemType, on_delete=models.CASCADE, null=True, default=None)

    # Fields that may or may not be present
    finish_date = models.DateTimeField(null=True, default=None)
    level_end_sp = models.IntegerField(null=True, default=None)
    level_start_sp = models.IntegerField(null=True, default=None)
    start_date = models.DateTimeField(null=True, default=None)
    training_start_sp = models.IntegerField(null=True, default=None)

    @property
    def sp_hour(self):
        return -1  # do some math


def valid_skills(value):
    try:
        data = json.loads(value)
        for skill, level in data.items():
            if not EveItemType.objects.filter(name=skill).exists():
                raise ValidationError(
                    _(f'Please enter a valid skill name for `{skill}`. Hint: a known character must have trained the skill for auth to know about it.')
                )
            lvl = -1
            try:
                lvl = int(level)
            except ValueError:
                pass
            if lvl > 5 or lvl < 0:
                raise ValidationError(
                    _(f'Please enter a valid skill level for `{skill}`. Hint: between 0 and 5.')
                )
    except ValueError:
        raise ValidationError(
            _('Please check format for valid JSON. Hint: ["skill name": 4, "skill name 2": 1]')
        )


class SkillList(models.Model):
    last_update = models.DateTimeField(auto_now=True)
    name = models.CharField(max_length=500, null=True, default=None)
    skill_list = models.TextField(
        null=True, default="", validators=[valid_skills])
    show_on_audit = models.BooleanField(default=True)
    order_weight = models.IntegerField(default=0)

    def get_skills(self):
        return json.loads(self.skill_list)

    def __str__(self):
        return "({}){} (Updated: {})".format(self.order_weight, self.name, self.last_update.strftime("%Y-%m-%d %H:%M:%S"))



from django.urls import include, path, re_path

from . import views
from .api import api

app_name = 'corptools'

urlpatterns = [
    path('', views.corptools_menu, name='view'),
    re_path(r'^api/', api.urls),
    path(
        'r/',
        views.react_menu_v4,
        name='react'
    ),
    re_path(
        r'^r/(?P<character_id>(\d)*)/',
        views.react_main,
        name='reactmain'
    ),
    re_path(
        'r/corp',
        views.react_corp_beta,
        name='corp_react'
    ),
    path(
        'r_legacy/',
        views.react_menu,
        name='reactlegacy'
    ),
    re_path(
        r'^r_legacy/(?P<character_id>(\d)*)/',
        views.v3_ui_render,
        name='reactlegacymain'
    ),
    re_path(r'^check_account/(?P<character_id>(\d)*)/$',
            views.update_account, name='update_account'),
    path('admin/', views.admin, name='admin'),
    path('admin_add_pyfa_xml/', views.admin_add_pyfa_xml,
         name='admin_add_pyfa_xml'),
    path('admin_create_tasks/', views.admin_create_tasks,
         name='admin_create_tasks'),
    path('run_tasks/', views.admin_run_tasks, name='run_tasks'),
    path('char/add/', views.add_char, name='add_char'),
    path('corp/', include([
        path('r_legacy/', views.react_corp, name='corp_legacy_react'),
        path('add/', views.add_corp, name='add_corp'),
        path('add_options/', views.add_corp_section,
             name='add_corp_options'),
        re_path(r'^dashboard/fuel', views.fuel_levels, name='fuel_dashboard'),
        re_path(r'^dashboard/metenox', views.metenox_levels,
                name='drill_dashboard'),
    ])),
]



# Generated by Django 2.2.12 on 2020-08-10 13:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0022_mapsystemgate_skilllist'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mapsystemgate',
            name='from_solar_system',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='mapsystemgate',
            name='to_solar_system',
            field=models.BigIntegerField(),
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-29 01:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0034_auto_20200928_0005'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charactermarketorder',
            name='escrow',
            field=models.DecimalField(
                decimal_places=2, default=None, max_digits=20, null=True),
        ),
        migrations.AlterField(
            model_name='corporationmarketorder',
            name='escrow',
            field=models.DecimalField(
                decimal_places=2, default=None, max_digits=20, null=True),
        ),
    ]



# Generated by Django 4.2.16 on 2025-08-01 09:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0124_characterbountyevent_characterbountystat"),
    ]

    operations = [
        migrations.AddField(
            model_name="corporationaudit",
            name="last_change_industry_jobs",
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="corporationaudit",
            name="last_update_industry_jobs",
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name="CorporationIndustryJob",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("activity_id", models.IntegerField()),
                ("blueprint_id", models.BigIntegerField()),
                ("blueprint_location_id", models.BigIntegerField()),
                ("blueprint_type_id", models.BigIntegerField()),
                (
                    "completed_character_id",
                    models.BigIntegerField(
                        blank=True, default=None, null=True),
                ),
                (
                    "completed_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                (
                    "cost",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        default=None,
                        max_digits=20,
                        null=True,
                    ),
                ),
                ("duration", models.IntegerField()),
                ("end_date", models.DateTimeField()),
                ("facility_id", models.BigIntegerField()),
                ("installer_id", models.IntegerField()),
                ("job_id", models.IntegerField()),
                (
                    "licensed_runs",
                    models.IntegerField(blank=True, default=None, null=True),
                ),
                ("location_id", models.BigIntegerField()),
                ("output_location_id", models.BigIntegerField()),
                (
                    "pause_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                ("probability", models.FloatField(
                    blank=True, default=None, null=True)),
                ("product_type_id", models.IntegerField()),
                ("runs", models.IntegerField()),
                ("start_date", models.DateTimeField()),
                ("status", models.CharField(max_length=15)),
                (
                    "successful_runs",
                    models.IntegerField(blank=True, default=None, null=True),
                ),
                (
                    "blueprint_type_name",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="corp_blueprint_type",
                        to="corptools.eveitemtype",
                    ),
                ),
                (
                    "corporation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="corptools.corporationaudit",
                    ),
                ),
                (
                    "product_type_name",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="corp_product_type",
                        to="corptools.eveitemtype",
                    ),
                ),
            ],
        ),
    ]



from model_utils import Choices

from django.db import models

from .audits import CharacterAudit, EveLocation
from .eve_models import EveItemType


class Clone(models.Model):
    character = models.OneToOneField(CharacterAudit, on_delete=models.CASCADE)

    last_clone_jump_date = models.DateTimeField(null=True, default=None)
    last_station_change_date = models.DateTimeField(null=True, default=None)
    location_id = models.BigIntegerField()
    location_name = models.ForeignKey(
        EveLocation, on_delete=models.SET_NULL, null=True, default=None)
    _type_enum = Choices('station', 'structure')
    location_type = models.CharField(max_length=9, choices=_type_enum)

# Clone Models


class JumpClone(models.Model):
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)

    jump_clone_id = models.IntegerField(null=True, default=None)
    location_id = models.BigIntegerField(null=True, default=None)
    location_name = models.ForeignKey(
        EveLocation, on_delete=models.SET_NULL, null=True, default=None)
    _type_enum = Choices('station', 'structure')
    location_type = models.CharField(
        max_length=9, choices=_type_enum, null=True, default=None)
    name = models.CharField(max_length=255, null=True, default=None)

# Implant Model


class Implant(models.Model):
    clone = models.ForeignKey(JumpClone, on_delete=models.CASCADE)
    type_name = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)



from django.apps import AppConfig

from . import __version__


class CorpToolsConfig(AppConfig):
    name = 'corptools'
    label = 'corptools'

    verbose_name = f"Corp Tools v{__version__}"

    def ready(self):
        from . import \
            signals  # NOQA - Loading Signals after models are loaded.



"""
This module is the core of the CorpTools Ecosystem
"""
__appname__ = "corptools"
__version__ = "3.0.0b5"
__url__ = "https://github.com/Solar-Helix-Independent-Transport/allianceauth-corp-tools"
__compat_date__ = "2025-09-30"



import json
import logging

from django.db import models

logger = logging.getLogger(__name__)


class JSONModel(models.Model):
    _filename = "not_set.jsonl"
    _update_fields = False

    @classmethod
    def from_jsonl(cls, json_data, name_lookup=False):
        raise AttributeError("Not Implemented")

    @classmethod
    def name_lookup(cls):
        return False

    @classmethod
    def create_update(cls, create_model_list: list["JSONModel"], update_model_list: list["JSONModel"]):
        cls.objects.bulk_create(
            create_model_list,
            ignore_conflicts=True
        )
        if cls._update_fields:
            cls.objects.bulk_update(
                update_model_list,
                cls._update_fields
            )

    @classmethod
    def load_from_sde(cls, folder_name):
        _creates = []
        _updates = []
        name_lookup = cls.name_lookup()
        pks = set(cls.objects.all().values_list("pk", flat=True)
                  ) if cls._update_fields else False
        file_path = f"{folder_name}/{cls._filename}"

        total_lines = 0
        with open(file_path, 'r') as json_file:
            while _ := json_file.readline():
                total_lines += 1

        total_read = 0
        with open(file_path, 'r') as json_file:
            while line := json_file.readline():
                rg = json.loads(line)
                _new = cls.from_jsonl(rg, name_lookup)
                if isinstance(_new, list):
                    if pks:
                        for _i in _new:
                            if _i.pk in pks:
                                _updates.append(_new)
                            else:
                                _creates.append(_new)
                    else:
                        _creates += _new
                else:
                    if pks:
                        if _new.pk in pks:
                            _updates.append(_new)
                        else:
                            _creates.append(_new)
                    else:
                        _creates.append(_new)

                total_read += 1

                if (len(_creates)+len(_updates)) >= 5000:
                    # lets batch these to reduce memory overhead
                    logger.info(
                        f"{file_path} - {int(total_read/total_lines*100)}% - {total_read}/{total_lines} Lines")
                    cls.create_update(_creates, _updates)
                    _creates = []
                    _updates = []

            # create/update any that are left.
            logger.info(
                f"{file_path} - {int(total_read/total_lines*100)}% - {total_read}/{total_lines} Lines")
            cls.create_update(_creates, _updates)

    class Meta:
        abstract = True



from typing import List

from ninja import NinjaAPI

from django.db.models import F, Sum
from django.utils.translation import gettext as _

from corptools import app_settings, models
from corptools.api import schema
from corptools.api.helpers import get_alts_queryset, get_main_character


class StatusApiEndpoints:

    tags = ["Account"]

    def __init__(self, api: NinjaAPI):

        @api.get(
            "account/{character_id}/status",
            response={200: schema.AccountStatus, 403: str},
            tags=self.tags
        )
        def get_character_status(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id
            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            char_skill_total = models.Skill.objects\
                .filter(character__character__in=characters)\
                .values('character')\
                .annotate(char=F('character__character__character_id'))\
                .annotate(total_sp=Sum('skillpoints_in_skill'))

            skills = {}

            for c in char_skill_total:
                skills[c.get('char')] = c.get('total_sp')

            characters = characters.select_related(
                'characteraudit', "characteraudit__location")
            output = {"characters": [],
                      "main": main
                      }
            for character in characters:
                last_login = None
                last_logout = None
                total_logins = 0
                if hasattr(character, "characteraudit"):
                    last_login = character.characteraudit.last_known_login
                    last_logout = character.characteraudit.last_known_logoff
                    total_logins = character.characteraudit.total_logins

                _o = {
                    "character": character,
                    "isk": 0,
                    "location": "Unknown",
                    "ship": "Unknown",
                    "ship_name": "Unknown",
                    "sp": skills.get(character.character_id, 0),
                    "active": False,
                    "last_updates": None,
                    "last_login": last_login,
                    "last_logout": last_logout,
                    "total_logins": total_logins
                }
                try:
                    _updates = {}
                    for grp in app_settings.get_character_update_attributes():
                        _updates[grp[0]] = getattr(
                            character.characteraudit, grp[1])
                    _o.update({
                        "isk": character.characteraudit.balance,
                        "active": character.characteraudit.is_active(),
                        "last_updates": _updates
                    })
                    try:
                        _o.update({
                            "location": character.characteraudit.location.current_location.location_name,
                            "ship": f"{character.characteraudit.location.current_ship.name}",
                            "ship_name": f"{character.characteraudit.location.current_ship_name}"
                        })
                    except Exception:
                        pass
                except models.CharacterAudit.DoesNotExist:
                    pass
                output["characters"].append(_o)

            output["characters"] = sorted(
                output["characters"], reverse=True, key=lambda d: d['sp']
            )
            return 200, output

        @api.get(
            "account/{character_id}/qs",
            tags=self.tags
        )
        def get_qs_test(request, character_id: int, total=500):
            if not request.user.is_superuser:
                return 403, _("Permission Denied")
            q = models.CharacterAudit.get_oldest_qs()

            out = []
            for s in q[:total]:
                out.append(
                    {
                        "Character": s.character.character_name,
                        "avg_date": s.avg_date
                    }
                )

            return out

        @api.get(
            "account/{character_id}/pubdata",
            response={200: List[schema.CharacterHistory], 403: str},
            tags=self.tags
        )
        def get_character_pubdata(request, character_id: int):
            if character_id == 0:
                character_id = request.user.profile.main_character.character_id

            response, main = get_main_character(request, character_id)

            if not response:
                return 403, _("Permission Denied")

            characters = get_alts_queryset(main)

            corp_histories = models.CorporationHistory.objects\
                .filter(character__character__in=characters)\
                .select_related(
                    'character__character', 'corporation_name'
                ).order_by("-record_id")

            histories = {}
            for h in corp_histories:
                if h.character.character_id not in histories:
                    histories[h.character.character_id] = []
                _h = {
                    "corporation": {
                        "corporation_name": h.corporation_name.name,
                        "corporation_id": h.corporation_name.eve_id,
                    },
                    "start": h.start_date
                }
                if h.corporation_name.alliance:
                    _h['corporation'].update({
                        "alliance_id": h.corporation_name.alliance.eve_id,
                        "alliance_name": h.corporation_name.alliance.name,
                    })
                histories[h.character.character_id].append(_h)

            characters = characters.select_related('characteraudit')
            output = []

            for character in characters:
                _o = {
                    "character": character,
                }
                try:
                    _o.update({
                        "history": histories[character.id],
                    })
                except KeyError:
                    pass
                output.append(_o)
            return 200, output



import json
import logging
from datetime import timedelta
from typing import List

from ninja import NinjaAPI

from django.db.models import F
from django.db.models.functions import Power, Sqrt
from django.http import HttpResponse
from django.utils import timezone

from allianceauth.services.hooks import get_extension_logger

from corptools import app_settings, models
from corptools.api import schema
from corptools.api.helpers import round_or_null

logger = get_extension_logger(__name__)


class StructureApiEndpoints:

    tags = ["Structures"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "corp/structures",
            response={200: List[schema.Structure], 403: str},
            tags=self.tags
        )
        def get_visible_structures(request):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view structures!")
                return 403, "Permission Denied!"

            output = []
            for s in models.Structure.get_visible(request.user).select_related(
                'type_name', "corporation__corporation", "system_name"
            ).prefetch_related('structureservice_set'):
                _ss = list()
                for __s in s.structureservice_set.all():
                    _ss.append({
                        "name": __s.name,
                        "state": __s.state
                    })
                _s = {
                    "id": s.structure_id,
                    "owner": s.corporation.corporation,
                    "name": s.name,
                    "type": {"id": s.type_id,
                             "name": s.type_name.name},
                    "services": _ss,
                    "location": {"id": s.system_name.system_id,
                                 "name": s.system_name.name},
                    "constellation": {"id": s.system_name.constellation.constellation_id,
                                      "name": s.system_name.constellation.name},
                    "region": {"id": s.system_name.constellation.region.region_id,
                               "name": s.system_name.constellation.region.name},
                    "fuel_expiry": s.fuel_expires,
                    "state": s.state,
                    "state_expiry": s.state_timer_end
                }
                output.append(_s)
            return list(output)

        @api.get(
            "corp/structures/fuel",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_fuel_data(request):
            if not (request.user.has_perm('corptools.own_corp_manager')
                    or request.user.has_perm('corptools.alliance_corp_manager')
                    or request.user.has_perm('corptools.state_corp_manager')
                    or request.user.has_perm('corptools.global_corp_manager')
                    or request.user.has_perm('corptools.holding_corp_structures')):
                logging.error(
                    f"Permission Denied for {request.user} to view Fuel!")
                return 403, "Permission Denied!"

            # hourly fuel reqs [ cit, eng, ref, flex ]
            citadel_service_mods = {
                'Clone Bay': [7.5, 10, 10, 10],
                'Market': [30, 40, 40, 40],
                'Manufacturing (Capitals)': [24, 18, 24, 24],
                # how to detect this
                'Standup Hyasyoda Research Lab': [10, 7.5, 10, 10],
                'Invention': [12, 9, 12, 12],
                'Manufacturing (Standard)': [12, 9, 12, 12],
                'Blueprint Copying': [12, 9, 12, 12],
                'Material Efficiency Research': [0, 0, 0, 0],  # part of above
                'Time Efficiency Research': [0, 0, 0, 0],  # part of above
                'Manufacturing (Super Capitals)': [36, 27, 36, 36],
                'Composite Reactions': [15, 15, 12, 15],
                'Hybrid Reactions': [15, 15, 12, 15],
                'Moon Drilling': [5, 5, 5, 5],
                'Biochemical Reactions': [15, 15, 12, 15],
                'Reprocessing': [10, 10, 7.5, 10],
                'Jump Access': [0, 0, 0, 15],  # large to show errors
                'Cynosural Jammer': [0, 0, 0, 40],
                'Jump Gate Access': [0, 0, 0, 30],
                'Automatic Moon Drilling': [0, 0, 0, 5]
            }

            cit = [35833, 47516, 47512, 47513, 47514, 47515, 35832, 35834]
            eng = [35827, 35826, 35825]
            ref = [35835, 35836]
            fle = [37534, 35841, 35840, 81826]

            flex_fuel_types = [81143]

            all_structures = models.Structure.get_visible(request.user).select_related(
                'corporation', 'corporation__corporation', 'system_name', 'type_name',
                'system_name__constellation', 'system_name__constellation__region'
            ).prefetch_related('structureservice_set')

            structure_tree = []
            total_hourly_fuel = 0
            for s in all_structures:
                structure_hourly_fuel = 0
                structure_type = 99

                if s.type_id in cit:
                    structure_type = 0
                elif s.type_id in eng:
                    structure_type = 1
                elif s.type_id in ref:
                    structure_type = 2
                elif s.type_id in fle:
                    structure_type = 3

                for service in s.structureservice_set.all():
                    if service.state == 'online':
                        fuel_use = citadel_service_mods[service.name][structure_type]
                        total_hourly_fuel += fuel_use
                        structure_hourly_fuel += fuel_use

                hours = 0

                if s.fuel_expires is not None:
                    hours = (s.fuel_expires - timezone.now()
                             ).total_seconds() // 3600

                extras = None

                if s.type_id == 81826:
                    fuels = models.CorpAsset.objects.filter(
                        type_id__in=flex_fuel_types,
                        location_id=s.structure_id
                    ).select_related(
                        'type_name'
                    )
                    out = {
                        "qty": 0,
                    }
                    for itm in fuels:
                        out["name"] = itm.type_name.name
                        out["qty"] += itm.quantity

                    out["expires"] = timezone.now() + timedelta(
                        hours=out["qty"] /
                        app_settings.CT_CHAR_METENOX_GAS_USE_HOURLY
                    )
                    if out['qty'] > 0:
                        extras = out

                structure_tree.append(
                    {
                        "id": s.structure_id,
                        "owner": {
                            "id": s.corporation.corporation.corporation_id,
                            "name": s.corporation.corporation.corporation_name
                        },
                        "name": s.name,
                        "type": {"id": s.type_id,
                                 "name": s.type_name.name},
                        "location": {"id": s.system_name.system_id,
                                     "name": s.system_name.name},
                        "constellation": {"id": s.system_name.constellation.constellation_id,
                                          "name": s.system_name.constellation.name},
                        "region": {"id": s.system_name.constellation.region.region_id,
                                   "name": s.system_name.constellation.region.name},
                        "fuel_expiry": s.fuel_expires,
                        "state": s.state,
                        "state_expiry": s.state_timer_end,
                        'fuel_req': structure_hourly_fuel,
                        "current_blocks": int(hours * structure_hourly_fuel),
                        "extra_fuel_info": extras,
                        "90day": max(
                            int(
                                (
                                    structure_hourly_fuel * 90 * 24
                                ) - (
                                    hours * structure_hourly_fuel
                                )
                            ),
                            0
                        )
                    }
                )

            context = {
                'structures': structure_tree,
                'total_hourly_fuel': total_hourly_fuel,
            }

            return context

        @api.get(
            "corp/structures/{structure_id}",
            response={200: dict, 404: str, 403: str},
            tags=self.tags
        )
        def get_corporation_structure_fitting(request, structure_id):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )
            structure = models.Structure.get_visible(request.user).filter(
                structure_id=structure_id)

            if not perms or not structure.exists():
                logging.error(
                    f"Permission Denied for {request.user} to view structures!")
                return 403, "Permission Denied!"

            output = {}
            assets = models.CorpAsset.objects.filter(location_id=structure_id)
            accepted_flags = [
                "Cargo",
                "StructureFuel",
                "QuantumCoreRoom",
                "FighterBay",
                "MoonMaterialBay"
            ]
            for a in assets:

                if "Slot" in a.location_flag:
                    output[a.location_flag] = {
                        "id": a.type_name.type_id,
                        "name": a.type_name.name
                    }
                elif "Tube" in a.location_flag:
                    output[a.location_flag] = {
                        "id": a.type_name.type_id,
                        "name": a.type_name.name
                    }
                else:
                    if a.location_flag in accepted_flags:
                        if a.location_flag not in output:
                            output[a.location_flag] = []
                        output[a.location_flag].append({
                            "id": a.type_name.type_id,
                            "name": a.type_name.name,
                            "qty": a.quantity
                        })

            _dogma = models.EveItemDogmaAttribute.objects.filter(
                eve_type_id=structure.first().type_id,
                attribute_id__in=[12, 13, 14, 1137, 2216, 2056]
            )
            slots = {"low": 8, "med": 8, "high": 8, "rig": 3,
                     "service": 0, "fighter": 0, "subsystem": 0}
            for d in _dogma:
                if d.attribute_id == 12:
                    slots["low"] = d.value
                if d.attribute_id == 13:
                    slots["med"] = d.value
                if d.attribute_id == 14:
                    slots["high"] = d.value
                if d.attribute_id == 1137:
                    slots["rig"] = d.value
                if d.attribute_id == 2216:
                    slots["fighter"] = d.value
                if d.attribute_id == 2056:
                    slots["service"] = d.value

            return {
                "fit": output,
                **slots
            }

        @api.get(
            "corp/pocos",
            response={200: List, 403: str},
            tags=self.tags
        )
        def get_visible_pocos(request):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view pocos!")
                return 403, "Permission Denied!"

            output = []
            for s in models.Poco.get_visible(request.user).select_related(
                "corporation__corporation", "system_name", "system_name__constellation", "system_name__constellation__region", "planet"
            ):
                try:
                    _s = {
                        "id": s.office_id,
                        "owner": {
                            "corporation_name": s.corporation.corporation.corporation_name,
                            "corporation_id": s.corporation.corporation.corporation_id
                        },
                        "name": s.name,
                        "location": {"id": s.planet.planet_id,
                                     "name": s.planet.name,
                                     "region": s.system_name.constellation.region.name,
                                     "constellation": s.system_name.constellation.name},
                        "standing_level": s.standing_level,
                        "alliance_tax_rate": round_or_null(s.alliance_tax_rate),
                        "corporation_tax_rate": round_or_null(s.corporation_tax_rate),
                        "terrible_standing_tax_rate": round_or_null(s.terrible_standing_tax_rate),
                        "bad_standing_tax_rate": round_or_null(s.bad_standing_tax_rate),
                        "neutral_standing_tax_rate": round_or_null(s.neutral_standing_tax_rate),
                        "good_standing_tax_rate": round_or_null(s.good_standing_tax_rate),
                        "excellent_standing_tax_rate": round_or_null(s.excellent_standing_tax_rate),
                        "reinforce_exit_end": s.reinforce_exit_end,
                        "reinforce_exit_start": s.reinforce_exit_start,
                        "allow_access_with_standings": s.allow_access_with_standings,
                        "allow_alliance_access": s.allow_alliance_access,
                    }
                    output.append(_s)
                except AttributeError:
                    logger.error(
                        f"Bad Poco {s}"
                    )
            return list(output)

        @api.get(
            "extra/jb/export",
            response={200: str, 403: str},
            tags=["Extras"]
        )
        def get_jb_link(request):
            output = []
            for s in models.Structure.objects.filter(
                type_id=35841
            ):
                id = s.structure_id
                split = s.name.split(" » ")

                from_sys = split[0]
                to_sys = split[1].split(" - ")[0]

                output.append(f"{id} {from_sys} --> {to_sys}")
            return HttpResponse("\n".join(output), content_type="text/plain")

        @api.get(
            "corp/starbases",
            response={200: List, 403: str},
            tags=self.tags
        )
        def get_visible_starbases(request):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view starbases!")
                return 403, "Permission Denied!"

            output = []
            for s in models.Starbase.get_visible(request.user).select_related(
                'type_name', "corporation__corporation", "system", "moon"
            ):
                extras = {}
                if s.moon:
                    extras["moon"] = {
                        "id": s.moon.moon_id,
                        "name": s.moon.name
                    }

                if s.system:
                    extras["system"] = {
                        "id": s.system.system_id,
                        "name": s.system.name
                    }
                    extras["constellation"] = {
                        "id": s.system.constellation.constellation_id,
                        "name": s.system.constellation.name
                    }
                    extras["region"] = {
                        "id": s.system.constellation.region.region_id,
                        "name": s.system.constellation.region.name
                    }

                _s = {
                    "starbase_id": s.starbase_id,
                    "owner": {
                        "corporation_id": s.corporation.corporation.corporation_id,
                        "corporation_name": s.corporation.corporation.corporation_name,
                    },
                    "name": s.name,
                    "type": {"id": s.type_name.type_id,
                             "name": s.type_name.name},
                    "state": s.state,
                    "onlined_since": s.onlined_since,
                    "reinforced_until": s.reinforced_until,
                    "unanchor_at": s.unanchor_at,
                    "allow_alliance_members": s.allow_alliance_members,
                    "allow_corporation_members": s.allow_corporation_members,
                    "attack_if_other_security_status_dropping": s.attack_if_other_security_status_dropping,
                    "use_alliance_standings": s.use_alliance_standings,
                    "attack_security_status_threshold": s.attack_security_status_threshold,
                    "attack_standing_threshold": s.attack_standing_threshold,
                    "anchor": s.anchor,
                    "online": s.online,
                    "offline": s.offline,
                    "unanchor": s.unanchor,
                    "fuel_bay_take": s.fuel_bay_take,
                    "fuel_bay_view": s.fuel_bay_view,
                    "fuels": s.fuels,
                    **extras
                }
                output.append(_s)
            return list(output)

        @api.get(
            "corp/starbase/{starbase_id}",
            response={200: dict, 403: str},
            tags=self.tags
        )
        def get_visible_starbase_fit(request, starbase_id: int):
            perms = (
                request.user.has_perm('corptools.own_corp_manager')
                | request.user.has_perm('corptools.alliance_corp_manager')
                | request.user.has_perm('corptools.state_corp_manager')
                | request.user.has_perm('corptools.global_corp_manager')
                | request.user.has_perm('corptools.holding_corp_structures')
            )

            if not perms:
                logging.error(
                    f"Permission Denied for {request.user} to view starbases!")
                return 403, "Permission Denied!"
            starbase = models.Starbase.get_visible(
                request.user
            ).filter(starbase_id=starbase_id)
            if starbase.exists():
                parent = models.CorpAsset.objects.filter(
                    item_id=starbase_id,
                    coordinate__isnull=False
                ).first()

                if parent:
                    logger.warning(f"Tower - {parent.type_name.name}")
                    distances = models.CorpAsset.objects.filter(
                        coordinate__isnull=False
                    ).annotate(
                        distance=Sqrt(
                            Power(
                                parent.coordinate.x - F("coordinate__x"),
                                2
                            ) + Power(
                                parent.coordinate.y - F("coordinate__y"),
                                2
                            ) + Power(
                                parent.coordinate.z - F("coordinate__z"),
                                2
                            )
                        )
                    ).filter(
                        distance__lte=100000
                    )

                    assets_in_space = []
                    for d in distances:
                        assets_in_space.append({
                            "type": {
                                "id": d.type_name.type_id,
                                "name": d.type_name.name
                            },
                            "distance": d.distance
                        })

                    assets_in_bay = []
                    starbase = starbase.first()
                    try:
                        for f in json.loads(starbase.fuels.replace("'", '"')):
                            # This is bad and i should feel bad....
                            # TODO save this correctly...
                            # TODO investigate if i can do this via assets betterer
                            type_name = models.EveItemType.objects.get(
                                type_id=f['type_id'])
                            assets_in_bay.append({
                                "id": type_name.type_id,
                                "name": type_name.name,
                                "qty": f['quantity']
                            })
                    except json.JSONDecodeError as e:
                        pass

                    return 200, {
                        "fuel": assets_in_bay,
                        "space": assets_in_space
                    }
            else:
                logging.error(
                    f"Permission Denied for {request.user} to view starbase {starbase_id}!")
                logger.warning(f"Tower - not found in assets")

                return 403, "Permission Denied!"



# Generated by Django 4.2.13 on 2024-05-27 07:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0103_currentshipfilter'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='currentshipfilter',
            name='categories',
        ),
        migrations.AlterField(
            model_name='currentshipfilter',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='Filter on Ship Groups.', to='corptools.eveitemgroup'),
        ),
        migrations.AlterField(
            model_name='currentshipfilter',
            name='types',
            field=models.ManyToManyField(blank=True, help_text='Filter on Ship Types.', to='corptools.eveitemtype'),
        ),
    ]



# Generated by Django 4.2.16 on 2025-02-25 12:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0114_alter_skilllist_skill_list'),
    ]

    operations = [
        migrations.CreateModel(
            name='AssetCoordiante',
            fields=[
                ('item', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE,
                 primary_key=True, related_name='coordinate', serialize=False, to='corptools.corpasset')),
                ('x', models.FloatField()),
                ('y', models.FloatField()),
                ('z', models.FloatField()),
            ],
        ),
    ]



from solo.admin import SingletonModelAdmin

from django.apps import apps
from django.contrib import admin
from django.utils.html import format_html

from . import app_settings, models

admin.site.register(models.CharacterAudit)
admin.site.register(models.CorporationAudit)


@admin.register(models.CorptoolsConfiguration)
class ConfigAdmin(SingletonModelAdmin):
    filter_horizontal = ["holding_corps"]


@admin.register(models.MapSystem)
class SystemAdmin(admin.ModelAdmin):
    list_display = ['name', 'get_region', 'get_constellation']
    search_fields = [
        'name', 'constellation__region__name', 'constellation__name']

    def get_region(self, obj):
        return obj.constellation.region.name

    def get_constellation(self, obj):
        return obj.constellation.name

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('constellation', 'constellation__region')

    def get_model_perms(self, request):
        return {}


@admin.register(models.MapRegion)
class MapRegionAdmin(admin.ModelAdmin):
    list_display = ['name']
    search_fields = ['name']

    def get_model_perms(self, request):
        return {}


@admin.register(models.MapConstellation)
class MapConstellationAdmin(admin.ModelAdmin):
    list_display = ['name', 'get_region']
    search_fields = ['name', 'region__name']

    def get_region(self, obj):
        return obj.region.name

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('region')

    def get_model_perms(self, request):
        return {}


@admin.register(models.MapSystemMoon)
class MapSystemMoonAdmin(admin.ModelAdmin):
    list_display = ['name', 'get_region', 'get_constellation', 'get_system']
    search_fields = ['name', 'system__constellation__region__name',
                     'system__constellation__name', 'system__name']

    def get_region(self, obj):
        return obj.system.constellation.region.name

    def get_constellation(self, obj):
        return obj.system.constellation.name

    def get_system(self, obj):
        return obj.system.name

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('system__constellation__region', 'system__constellation', 'system')

    def get_model_perms(self, request):
        return {}


@admin.register(models.MapSystemPlanet)
class PlanetSystemMoonAdmin(admin.ModelAdmin):
    list_display = ['name', 'get_region', 'get_constellation', 'get_system']
    search_fields = ['name', 'system__constellation__region__name',
                     'system__constellation__name', 'system__name']

    def get_region(self, obj):
        return obj.system.constellation.region.name

    def get_constellation(self, obj):
        return obj.system.constellation.name

    def get_system(self, obj):
        return obj.system.name

    def get_queryset(self, request):
        return super().get_queryset(request).select_related('system__constellation__region', 'system__constellation', 'system')

    def get_model_perms(self, request):
        return {}


@admin.register(models.EveName)
class EveNameAdmin(admin.ModelAdmin):
    search_fields = ['name']


@admin.register(models.CorporationWalletJournalEntry)
class CorporationWalletJournalEntryAdmin(admin.ModelAdmin):
    list_display = ['date', 'first_party_name', 'second_party_name',
                    'entry_id', 'ref_type', 'reason', 'amount']
    search_fields = ['description', 'entry_id', 'reason',
                     'first_party_name__name', 'second_party_name__name', ]
    list_filter = ['ref_type']
    list_select_related = (
        'first_party_name', 'second_party_name',
    )


@admin.register(models.SkillList)
class SkillListAdmin(admin.ModelAdmin):
    list_display = ['order_weight', 'name', 'last_update']
    search_fields = ['name', 'skill_list', ]


@admin.register(models.MapJumpBridge)
class BridgeAdmin(admin.ModelAdmin):

    list_select_related = (
        'from_solar_system',
        'to_solar_system',
    )

    autocomplete_fields = ['from_solar_system', 'to_solar_system', 'owner']


@admin.register(models.CharacterTitle)
class TitleAdmin(admin.ModelAdmin):
    list_display = ['title', 'corporation_name']
    search_fields = ['title', 'corporation_name']
    list_filter = ['corporation_name']

    def get_model_perms(self, request):
        return {}


class assetFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', '_types', '_groups', '_cats',
                    '_systems', '_constellations', '_regions']

    def _list_2_html_w_tooltips(self, my_items: list, max_items: int) -> str:
        """converts list of strings into HTML with cutoff and tooltip"""
        items_truncated_str = ', '.join(my_items[:max_items])
        if not my_items:
            result = None
        elif len(my_items) <= max_items:
            result = items_truncated_str
        else:
            items_truncated_str += ', (...)'
            items_all_str = ', '.join(my_items)
            result = format_html(
                '<span data-tooltip="{}" class="tooltip">{}</span>',
                items_all_str,
                items_truncated_str
            )
        return result

    @admin.display(
        description='types'
    )
    def _types(self, obj):
        my_types = [x.name for x in obj.types.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_types,
            10
        )

    @admin.display(
        description='groups'
    )
    def _groups(self, obj):
        my_groups = [x.name for x in obj.groups.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_groups,
            10
        )

    @admin.display(
        description='categories'
    )
    def _cats(self, obj):
        my_cats = [x.name for x in obj.categories.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_cats,
            10
        )

    @admin.display(
        description='systems'
    )
    def _systems(self, obj):
        my_systems = [x.name for x in obj.systems.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_systems,
            10
        )

    @admin.display(
        description='constellations'
    )
    def _constellations(self, obj):
        my_constels = [x.name for x in obj.constellations.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_constels,
            10
        )

    @admin.display(
        description='regions'
    )
    def _regions(self, obj):
        my_regions = [x.name for x in obj.regions.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_regions,
            10
        )

    filter_horizontal = ["types",
                         "groups",
                         "categories",
                         "systems",
                         "constellations",
                         "regions"]


class CurrentShipFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', '_types', '_groups',
                    '_systems', '_constellations', '_regions']
    filter_horizontal = ["types", "groups",
                         "systems", "constellations", "regions"]

    def _list_2_html_w_tooltips(self, my_items: list, max_items: int) -> str:
        """converts list of strings into HTML with cutoff and tooltip"""
        items_truncated_str = ', '.join(my_items[:max_items])
        if not my_items:
            result = None
        elif len(my_items) <= max_items:
            result = items_truncated_str
        else:
            items_truncated_str += ', (...)'
            items_all_str = ', '.join(my_items)
            result = format_html(
                '<span data-tooltip="{}" class="tooltip">{}</span>',
                items_all_str,
                items_truncated_str
            )
        return result

    @admin.display(description='types')
    def _types(self, obj):
        my_types = [x.name for x in obj.types.order_by('name')]

        return self._list_2_html_w_tooltips(my_types, 10)

    @admin.display(description='groups')
    def _groups(self, obj):
        my_groups = [x.name for x in obj.groups.order_by('name')]

        return self._list_2_html_w_tooltips(my_groups, 10)

    @admin.display(description='systems')
    def _systems(self, obj):
        my_systems = [x.name for x in obj.systems.order_by('name')]

        return self._list_2_html_w_tooltips(my_systems, 10)

    @admin.display(description='constellations')
    def _constellations(self, obj):
        my_constellations = [
            x.name for x in obj.constellations.order_by('name')]

        return self._list_2_html_w_tooltips(my_constellations, 10)

    @admin.display(description='regions')
    def _regions(self, obj):
        my_regions = [x.name for x in obj.regions.order_by('name')]

        return self._list_2_html_w_tooltips(my_regions, 10)


class skillsFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', '_required_skill_lists',
                    '_single_req_skill_lists']
    filter_horizontal = ["required_skill_lists",
                         "single_req_skill_lists"]

    def _list_2_html_w_tooltips(self, my_items: list, max_items: int) -> str:
        """converts list of strings into HTML with cutoff and tooltip"""
        items_truncated_str = ', '.join(my_items[:max_items])
        if not my_items:
            result = None
        elif len(my_items) <= max_items:
            result = items_truncated_str
        else:
            items_truncated_str += ', (...)'
            items_all_str = ', '.join(my_items)
            result = format_html(
                '<span data-tooltip="{}" class="tooltip">{}</span>',
                items_all_str,
                items_truncated_str
            )
        return result

    @admin.display(
        description='Require One'
    )
    def _single_req_skill_lists(self, obj):
        my_single_req_skill_lists = [
            x.name for x in obj.single_req_skill_lists.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_single_req_skill_lists,
            10
        )

    @admin.display(
        description='Required'
    )
    def _required_skill_lists(self, obj):
        my_required_skill_lists = [
            x.name for x in obj.required_skill_lists.order_by('name')]

        return self._list_2_html_w_tooltips(
            my_required_skill_lists,
            10
        )


class TimeInCorpFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'days_in_corp', "reversed_logic"]


class CharacterAgeFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'min_age', "reversed_logic"]


class rolesFilterAdmin(admin.ModelAdmin):
    filter_horizontal = ["corps_filter",
                         "alliances_filter"]

    list_display = ['__str__', 'has_director', 'has_accountant',
                    'has_station_manager', 'has_personnel_manager']


class titleFilterAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'titles']


class LoginAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'days_since_login',
                    'no_data_pass', 'main_corp_only']


class HomeStationFilterAdmin(admin.ModelAdmin):
    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == "evelocation":
            data = set()
            for f in models.EveLocation._meta.get_fields():
                if f.auto_created and not f.concrete:
                    data.update(
                        set(
                            f.related_model.objects.all().values_list(
                                f.field.attname, flat=True).distinct()
                        )
                    )
            kwargs["queryset"] = models.EveLocation.objects.filter(
                location_id__in=list(data),
                managed=False
            )
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    filter_horizontal = ["evelocation"]


class JumpCloneFilterAdmin(admin.ModelAdmin):
    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == "evelocation":
            data = set()
            for f in models.EveLocation._meta.get_fields():
                if f.auto_created and not f.concrete:
                    print(f)
                    data.update(
                        set(
                            f.related_model.objects.all().values_list(
                                f.field.attname, flat=True).distinct()
                        )
                    )
            kwargs["queryset"] = models.EveLocation.objects.filter(
                location_id__in=list(data),
                managed=False
            )
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    filter_horizontal = ["evelocation"]


if apps.is_installed('securegroups'):
    admin.site.register(models.FullyLoadedFilter)
    admin.site.register(models.TimeInCorpFilter, TimeInCorpFilterAdmin)
    admin.site.register(models.CharacterAgeFilter, CharacterAgeFilterAdmin)
    if app_settings.CT_CHAR_ASSETS_MODULE:
        admin.site.register(models.AssetsFilter, assetFilterAdmin)
    if app_settings.CT_CHAR_LOCATIONS_MODULE:
        admin.site.register(models.CurrentShipFilter, CurrentShipFilterAdmin)
    if app_settings.CT_CHAR_SKILLS_MODULE:
        admin.site.register(models.Skillfilter, skillsFilterAdmin)
    if app_settings.CT_CHAR_ROLES_MODULE:
        admin.site.register(models.Titlefilter, titleFilterAdmin)
        admin.site.register(models.Rolefilter, rolesFilterAdmin)
    if app_settings.CT_CHAR_CLONES_MODULE:
        admin.site.register(models.HomeStationFilter, HomeStationFilterAdmin)
        admin.site.register(models.JumpCloneFilter, JumpCloneFilterAdmin)
    admin.site.register(models.HighestSPFilter)
    admin.site.register(models.LastLoginfilter, LoginAdmin)



# Generated by Django 4.2.16 on 2026-02-08 08:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0128_clear_skill_cache"),
    ]

    operations = [
        migrations.AlterField(
            model_name="notification",
            name="id",
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 4.2.23 on 2025-09-15 16:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0125_corporationaudit_last_change_industry_jobs_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='assetsfilter',
            name='reversed_logic',
            field=models.BooleanField(default=False, help_text='Negate the value of the filter, i.e. check for absence of assets'),
        ),
    ]



from typing import List

from ninja import NinjaAPI

from django.db.models import Q

from esi.models import Token

from corptools import app_settings, models
from corptools.api import schema
from corptools.tasks import update_corp


class ListApiEndpoints:

    tags = ["Corporation"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "corp/list",
            response={200: List[schema.CorpStatus], 403: str},
            tags=self.tags
        )
        def get_visible_corporation_status(request):
            corps = models.CorporationAudit.objects.visible_to(
                request.user
            ).filter(
                corporation__corporation_id__gte=2000000
            )

            if (request.user.has_perm("corptools.holding_corp_wallets")
                or request.user.has_perm("corptools.holding_corp_assets")
                    or request.user.has_perm("corptools.holding_corp_structures")):
                corps_holding = models.CorptoolsConfiguration.objects.get(
                    id=1).holding_corp_qs()
                corps = corps | corps_holding

            chars = models.CharacterAudit.objects.filter(
                character__corporation_id__in=corps.values_list(
                    "corporation__corporation_id",
                    flat=True
                ),
                active=True
            )
            chars = chars.select_related(
                "characterroles",
                "character"
            ).filter(
                Q(characterroles__accountant=True)
                or Q(characterroles__director=True)
                or Q(characterroles__station_manager=True)
            )

            corp_chars = {}

            for c in corps:
                corp_chars[c.corporation.corporation_id] = {
                    "a": {
                        "c": 0,
                        "t": 0
                    },
                    "w": {
                        "c": 0,
                        "t": 0
                    },
                    "s": {
                        "c": 0,
                        "t": 0
                    },
                    "m": {
                        "c": 0,
                        "t": 0
                    },
                }

            _c = {}
            for c in chars:
                _c[c.character.character_id] = c.character.corporation_id
                if c.characterroles.director:
                    corp_chars[c.character.corporation_id]["a"]["c"] += 1
                    corp_chars[c.character.corporation_id]["w"]["c"] += 1
                    corp_chars[c.character.corporation_id]["s"]["c"] += 1
                    corp_chars[c.character.corporation_id]["m"]["c"] += 1
                else:
                    if c.characterroles.station_manager:
                        corp_chars[c.character.corporation_id]["s"]["c"] += 1
                    if c.characterroles.accountant:
                        corp_chars[c.character.corporation_id]["w"]["c"] += 1
                        corp_chars[c.character.corporation_id]["m"]["c"] += 1

            # TODO rethink how we do this Issue #121

            # tokens = Token.objects.filter(character_id__in=chars.values_list(
            #     "character__character_id", flat=True))

            # def token_scope_filter(qs, scopes):
            #     for s in scopes:
            #         qs = qs.filter(scopes__name=s)
            #     return qs

            # def filter_token(qs, grp):
            #     for t in qs:
            #         corp_chars[_c[t.character_id]][grp]["t"] += 1

            # a_tokens = token_scope_filter(
            #     tokens, app_settings._corp_scopes_base + app_settings._corp_scopes_assets)
            # filter_token(a_tokens, "a")
            # w_tokens = token_scope_filter(
            #     tokens, app_settings._corp_scopes_base + app_settings._corp_scopes_wallets)
            # filter_token(w_tokens, "w")
            # s_tokens = token_scope_filter(
            #     tokens, app_settings._corp_scopes_base + app_settings._corp_scopes_structures)
            # filter_token(s_tokens, "s")
            # m_tokens = token_scope_filter(
            #     tokens, app_settings._corp_scopes_base + app_settings._corp_scopes_moons)
            # filter_token(m_tokens, "m")

            output = []
            for c in corps:
                _updates = {}
                for grp in app_settings.get_corp_update_attributes():
                    _updates[grp[0]] = {
                        "update": getattr(c, grp[1]),
                        "chars": corp_chars[c.corporation.corporation_id][grp[2]]["c"],
                        "tokens": corp_chars[c.corporation.corporation_id][grp[2]]["t"]
                    }
                all_id = None
                all_nm = None
                if c.corporation.alliance:
                    all_id = c.corporation.alliance.alliance_id
                    all_nm = c.corporation.alliance.alliance_name

                _out = {
                    "corporation": {
                        "corporation_id": c.corporation.corporation_id,
                        "corporation_name": c.corporation.corporation_name,
                        "alliance_id": all_id,
                        "alliance_name": all_nm
                    },
                    "characters": c.corporation.member_count,
                    "active": True,
                    "last_updates": _updates
                }
                output.append(_out)
            return output

        @api.post(
            "corporation/refresh",
            response={200: schema.Message, 403: str},
            tags=self.tags
        )
        def post_corp_refresh(request, corporation_id: int):
            corp = models.CorporationAudit.objects.visible_to(
                request.user
            ).filter(
                corporation__corporation_id=corporation_id
            )

            if not corp.exists():
                return 403, _("Permission Denied")

            force = app_settings.CT_USERS_CAN_FORCE_REFRESH or request.user.is_superuser

            update_corp.apply_async(
                args=[corporation_id],
                kwargs={
                    "force_refresh": force
                },
                priority=4)
            return 200, {"message": "Requested Updates!"}



# Generated by Django 3.2.8 on 2022-01-09 01:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0060_bridgeozonelevel'),
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationText',
            fields=[
                ('notification_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('notification_text', models.TextField(default=None, null=True)),
            ],
        ),
    ]



# Generated by Django 3.1.1 on 2021-01-12 13:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0040_auto_20210112_0449'),
    ]

    operations = [
        migrations.CreateModel(
            name='TimeInCorpFilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('days_in_corp', models.IntegerField(default=30)),
            ],
            options={
                'verbose_name': "Smart Filter: Main's Time in Corp",
                'verbose_name_plural': "Smart Filter: Main's Time in Corp",
            },
        ),
        migrations.AlterModelOptions(
            name='assetsfilter',
            options={'verbose_name': 'Smart Filter: Assets in Locations',
                     'verbose_name_plural': 'Smart Filter: Assets in Locations'},
        ),
        migrations.AlterField(
            model_name='skillfilter',
            name='required_skill_lists',
            field=models.ManyToManyField(blank=True, to='corptools.SkillList'),
        ),
    ]



# Generated by Django 4.2.16 on 2025-11-09 02:00

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0126_assetsfilter_reversed_logic"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="corporationaudit",
            options={
                "permissions": (
                    ("own_corp_manager", "Can access own corporations's data."),
                    (
                        "alliance_corp_manager",
                        "Can access other corporations's data for own alliance.",
                    ),
                    (
                        "state_corp_manager",
                        "Can access other corporations's data for own state.",
                    ),
                    ("global_corp_manager", "Can access all corporations's data."),
                    (
                        "show_if_director",
                        "Can access all corporations's where character is a director.",
                    ),
                )
            },
        ),
    ]



from typing import List

from ninja import NinjaAPI

from django.utils.translation import gettext as _

from ... import app_settings
from .. import schema


class MenuApiEndpoints:

    tags = ["Menus"]

    def __init__(self, api: NinjaAPI):

        @api.get(
            "account/menu",
            response=List[schema.MenuCategory],
            tags=self.tags
        )
        def get_character_menu(request):
            _inter = {
                "name": _("Interactions"),
                "links": []
            }
            _finance = {
                "name": _("Finances"),
                "links": []
            }
            _industry = {
                "name": _("Industry"),
                "links": []
            }
            _char = {
                "name": _("Characters"),
                "links": [
                    {
                        "name": _("Status"),
                        "link": "account/status"
                    },
                    {
                        "name": _("Corp History"),
                        "link": "account/pubdata"
                    }
                ]
            }

            if app_settings.CT_CHAR_CONTACTS_MODULE:
                _inter["links"].append({
                    "name": _("Contact"),
                    "link": "account/contact"
                })

            if app_settings.CT_CHAR_NOTIFICATIONS_MODULE:
                _inter["links"].append({
                    "name": _("Notifications"),
                    "link": "account/notifications"
                })

            # if app_settings.CT_CHAR_STANDINGS_MODULE:
            #    _inter["links"].append({
            #        "name": "Standings",
            #        "link": "/account/standings"
            #    })

            if app_settings.CT_CHAR_WALLET_MODULE:
                _finance["links"].append({
                    "name": _("Wallet"),
                    "link": "account/wallet"
                })
                if (request.user.has_perm("corptools.global_corp_manager")
                    or request.user.has_perm("corptools.state_corp_manager")
                    or request.user.has_perm("corptools.alliance_corp_manager")
                        or request.user.has_perm("corptools.own_corp_manager")):
                    _finance["links"].append({
                        "name": _("Wallet Activity"),
                        "link": "account/walletactivity"
                    })
                _finance["links"].append({
                    "name": _("Contracts"),
                    "link": "account/contract"
                })

                _finance["links"].append({
                    "name": _("Market"),
                    "link": "account/market"
                })

            if app_settings.CT_CHAR_LOYALTYPOINTS_MODULE:
                _finance["links"].append({
                    "name": _("Loyalty Points"),
                    "link": "account/lp"
                })

            if app_settings.CT_CHAR_ASSETS_MODULE:
                _char["links"].append({
                    "name": _("Asset Overview"),
                    "link": "account/assets"
                })
                _char["links"].append({
                    "name": _("Asset List"),
                    "link": "account/listassets"
                })

            if app_settings.CT_CHAR_MINING_MODULE:
                _industry["links"].append({
                    "name": _("Mining Ledger"),
                    "link": "account/mining"
                })

            if app_settings.CT_CHAR_CLONES_MODULE:
                _char["links"].append({
                    "name": _("Clones"),
                    "link": "account/clones"
                })

            if app_settings.CT_CHAR_ROLES_MODULE:
                _char["links"].append({
                    "name": _("Roles"),
                    "link": "account/roles"
                })

            if app_settings.CT_CHAR_MAIL_MODULE:
                _inter["links"].append({
                    "name": _("Mail"),
                    "link": "account/mail"
                })

            if app_settings.CT_CHAR_SKILLS_MODULE:
                _char["links"].append({
                    "name": _("Skills"),
                    "link": "account/skills"
                })
                _char["links"].append({
                    "name": _("Skill Queues"),
                    "link": "account/skillqueue"
                })
                _char["links"].append({
                    "name": _("Skill List Checks"),
                    "link": "account/doctrines"
                })
                _char["links"].append({
                    "name": _("Fitting Skill Checks"),
                    "link": "account/fitcheck"
                })
            admin = {
                "name": _("Admin"),
                "link": "/audit/admin"
            }
            out = []
            if len(_char['links']):
                out.append(_char)

            if len(_finance['links']):
                out.append(_finance)

            if len(_industry['links']):
                out.append(_industry)

            if len(_inter['links']):
                out.append(_inter)

            if request.user.is_superuser:
                out.append(admin)

            return out



# Generated by Django 3.2.5 on 2021-08-27 12:38

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0058_auto_20210808_0635'),
    ]

    operations = [
        migrations.AddField(
            model_name='invtypematerials',
            name='met_type',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    related_name='met_type', to='corptools.eveitemtype'),
        ),
    ]



from typing import TYPE_CHECKING

from model_utils import Choices

from django.db import models

from .audits import CharacterAudit, CorporationAudit, CorptoolsConfiguration
from .eve_models import EveItemType, EveName

if TYPE_CHECKING:
    from esi.stubs import CharactersCharacterIdWalletJournalGetItem


class WalletJournalEntry(models.Model):
    amount = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    balance = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    context_id = models.BigIntegerField(null=True, default=None)
    _context_type_enum = Choices('structure_id', 'station_id', 'market_transaction_id', 'character_id',
                                 'corporation_id', 'alliance_id', 'eve_system', 'industry_job_id',
                                 'contract_id', 'planet_id', 'system_id', 'type_id')
    context_id_type = models.CharField(
        max_length=30, choices=_context_type_enum, null=True, default=None)
    date = models.DateTimeField()
    description = models.CharField(max_length=500)
    first_party_id = models.IntegerField(null=True, default=None)
    entry_id = models.BigIntegerField()
    reason = models.CharField(max_length=500, null=True, default=None)
    ref_type = models.CharField(max_length=72)
    second_party_id = models.IntegerField(null=True, default=None)
    tax = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    tax_receiver_id = models.IntegerField(null=True, default=None)

    processed = models.BooleanField(default=False)

    class Meta:
        abstract = True
        indexes = (
            models.Index(fields=['date']),
            models.Index(fields=['entry_id']),
            models.Index(fields=['ref_type'])
        )


class CorporationWalletDivision(models.Model):
    corporation = models.ForeignKey(
        CorporationAudit, on_delete=models.CASCADE, related_name='corporation_division')
    name = models.CharField(max_length=100, null=True, default=None)
    balance = models.DecimalField(max_digits=20, decimal_places=2)
    division = models.IntegerField()

    @classmethod
    def get_visible(cls, user):
        corps_vis = CorporationAudit.objects.visible_to(user)
        if user.has_perm("corptools.holding_corp_wallets"):
            corps_holding = CorptoolsConfiguration.get_solo().holding_corp_qs()
            corps_vis = corps_vis | corps_holding

        return cls.objects.filter(corporation__in=corps_vis)


class CorporationWalletJournalEntry(WalletJournalEntry):
    division = models.ForeignKey(
        CorporationWalletDivision, on_delete=models.CASCADE)
    first_party_name = models.ForeignKey(
        EveName, on_delete=models.SET_NULL, null=True, default=None, related_name='corp_first_party')
    second_party_name = models.ForeignKey(
        EveName, on_delete=models.SET_NULL, null=True, default=None, related_name='corp_second_party')

    def __str__(self):
        return "{} '{}' {}: {}isk".format(
            self.first_party_name.name,
            self.ref_type,
            self.second_party_name.name,
            self.amount
        )

    @classmethod
    def get_visible(cls, user):
        corps_vis = CorporationAudit.objects.visible_to(user)
        if user.has_perm("corptools.holding_corp_wallets"):
            corps_holding = CorptoolsConfiguration.get_solo().holding_corp_qs()
            corps_vis = corps_vis | corps_holding

        return cls.objects.filter(division__corporation__in=corps_vis)


class CharacterWalletJournalEntry(WalletJournalEntry):
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)
    first_party_name = models.ForeignKey(
        EveName, on_delete=models.SET_NULL, null=True, default=None, related_name='char_first_party')
    second_party_name = models.ForeignKey(
        EveName, on_delete=models.SET_NULL, null=True, default=None, related_name='char_second_party')

    @classmethod
    def from_esi_model(cls, character: CharacterAudit, esi_model: "CharactersCharacterIdWalletJournalGetItem"):
        return cls(
            character=character,
            amount=esi_model.amount,
            balance=esi_model.balance,
            context_id=esi_model.context_id,
            context_id_type=esi_model.context_id_type,
            date=esi_model.date,
            description=esi_model.description,
            first_party_id=esi_model.first_party_id,
            first_party_name_id=esi_model.first_party_id,
            entry_id=esi_model.id,
            reason=esi_model.reason,
            ref_type=esi_model.ref_type,
            second_party_id=esi_model.second_party_id,
            second_party_name_id=esi_model.second_party_id,
            tax=esi_model.tax,
            tax_receiver_id=esi_model.tax_receiver_id,
        )


class CharacterBountyEvent(models.Model):
    entry = models.OneToOneField(
        CharacterWalletJournalEntry,
        on_delete=models.CASCADE,
        related_name="msg"
    )
    message = models.TextField()


class CharacterBountyStat(models.Model):
    event = models.ForeignKey(
        CharacterBountyEvent,
        on_delete=models.CASCADE,
        related_name="stats"
    )
    type_name = models.ForeignKey(
        EveItemType,
        on_delete=models.CASCADE
    )
    qty = models.IntegerField()



from datetime import timedelta

import yaml
from ninja import NinjaAPI

from django.utils import timezone

from ... import models


class CloneDeathApiEndpoints:
    tags = ["Extras"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "/extras/clone_death/{location_id}/{look_back}/",
            response={200: dict, 403: str, 404: str, 500: str},
            tags=self.tags
        )
        def get_dead_clones_in_location(request, location_id: int, look_back: int):
            """
                Find Dead Clones :pepewhy:
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            out = {"m": {}, "i": {}}
            since = timezone.now() - timedelta(hours=look_back)

            notes = models.Notification.objects.filter(
                notification_type="JumpCloneDeletedMsg1", timestamp__gte=since)

            for n in notes:
                ny = yaml.load(
                    n.notification_text.notification_text, Loader=yaml.UnsafeLoader)
                if len(ny["typeIDs"]) and ny['locationID'] == location_id:
                    try:
                        mc = n.character.character.character_ownership.user.profile.main_character.character_name
                    except Exception:
                        continue
                    cc = n.character.character.character_name
                    if mc not in out["m"]:
                        out["m"][mc] = {"c": [], "i": {}}
                    if cc not in out["m"][mc]["c"]:
                        out["m"][mc]["c"].append(cc)
                    for ti in ny["typeIDs"]:
                        nm = models.EveItemType.objects.get(type_id=ti).name
                        if ti not in out["m"][mc]["i"]:
                            out["m"][mc]["i"][ti] = {"c": 0, "n": nm}
                        out["m"][mc]["i"][ti]["c"] += 1
                        if ti not in out["i"]:
                            out["i"][ti] = {"c": 0, "n": nm}
                        out["i"][ti]["c"] += 1

            return out



# Generated by Django 4.2.16 on 2024-09-23 00:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0110_rename_oreprice_typeprice'),
    ]

    operations = [
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_assets',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Asset Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_clones',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Clone Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_contacts',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Contact Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_contracts',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Contracts Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_indy',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character industry Job Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_location',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Location Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_loyaltypoints',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Loyalty Point Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_mails',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Mail Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_mining',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Mining Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_notif',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Notification Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_pub_data',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Pulic History Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_roles',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Roles Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_skills',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character SKill Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_wallet',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Wallet Data'),
        ),
    ]



from typing import TYPE_CHECKING, Union

from model_utils import Choices

from django.db import models

from .audits import CharacterAudit, EveLocation
from .eve_models import EveItemType, MapRegion
from .wallets import CorporationWalletDivision

if TYPE_CHECKING:
    from esi.stubs import (
        CharactersCharacterIdOrdersGetItem,
        CharactersCharacterIdOrdersHistoryGetItem,
    )


class MarketOrder(models.Model):
    order_id = models.BigIntegerField(primary_key=True)

    duration = models.IntegerField()
    escrow = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    is_buy_order = models.BooleanField(null=True, default=None)
    issued = models.DateTimeField()
    location_id = models.BigIntegerField()
    location_name = models.ForeignKey(
        EveLocation, on_delete=models.SET_NULL, null=True, default=None)

    min_volume = models.IntegerField(null=True, default=None)
    price = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    _range_enum = Choices('1', '10', '2', '20', '3', '30',
                          '4', '40', '5', 'region', 'solarsystem', 'station')
    order_range = models.CharField(max_length=30, choices=_range_enum)
    region_id = models.IntegerField()
    region_name = models.ForeignKey(
        MapRegion, on_delete=models.SET_NULL, null=True, default=None)
    type_id = models.IntegerField()
    type_name = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)
    volume_remain = models.IntegerField()
    volume_total = models.IntegerField()

    _state_enum = Choices('cancelled', 'expired', 'active')
    state = models.CharField(max_length=30, choices=_state_enum)

    class Meta:
        abstract = True


class CorporationMarketOrder(MarketOrder):
    wallet_division = models.ForeignKey(
        CorporationWalletDivision, on_delete=models.CASCADE)
    issued_by = models.IntegerField()


class CharacterMarketOrder(MarketOrder):
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)
    is_corporation = models.BooleanField()

    @classmethod
    def from_esi_model(
        cls,
        character: CharacterAudit,
        esi_model: Union[
            "CharactersCharacterIdOrdersGetItem",
            "CharactersCharacterIdOrdersHistoryGetItem"
        ]
    ):
        return cls(
            character=character,
            order_id=esi_model.order_id,
            duration=esi_model.duration,
            escrow=esi_model.escrow,
            is_buy_order=esi_model.is_buy_order,
            issued=esi_model.issued,
            location_id=esi_model.location_id,
            min_volume=esi_model.min_volume,
            price=esi_model.price,
            order_range=esi_model.range,
            region_id=esi_model.region_id,
            region_name_id=esi_model.region_id,
            type_id=esi_model.type_id,
            type_name_id=esi_model.type_id,
            volume_remain=esi_model.volume_remain,
            volume_total=esi_model.volume_total,
            is_corporation=esi_model.is_corporation,
            state=getattr(esi_model, "state", "active")
        )



from celery.schedules import crontab
from django_celery_beat.models import CrontabSchedule, PeriodicTask

from django.core.management.base import BaseCommand

from allianceauth.crontab.utils import offset_cron

from corptools.tasks import update_models_from_sde


class Command(BaseCommand):
    help = 'Bootstrap the CorpTools Module'

    def add_arguments(self, parser):
        parser.add_argument('--inline', action='store_true',
                            help='Run update in this Console not via Celery')

    def handle(self, *args, **options):
        self.stdout.write("Configuring Tasks!")

        schedule_a = CrontabSchedule.from_schedule(
            offset_cron(crontab(minute='15,45')))
        schedule_char, created = CrontabSchedule.objects.get_or_create(
            minute=schedule_a.minute,
            hour=schedule_a.hour,
            day_of_month=schedule_a.day_of_month,
            month_of_year=schedule_a.month_of_year,
            day_of_week=schedule_a.day_of_week,
            timezone=schedule_a.timezone,
        )

        schedule_b = CrontabSchedule.from_schedule(
            offset_cron(crontab(minute='30', hour='*')))
        schedule_corp, schedule_corp_created = CrontabSchedule.objects.get_or_create(
            minute=schedule_b.minute,
            hour=schedule_b.hour,
            day_of_month=schedule_b.day_of_month,
            month_of_year=schedule_b.month_of_year,
            day_of_week=schedule_b.day_of_week,
            timezone=schedule_b.timezone,
        )

        PeriodicTask.objects.update_or_create(
            task='corptools.tasks.update_subset_of_characters',
            defaults={
                'crontab': schedule_char,
                'name': 'Character Audit Rolling Update',
                'enabled': True
            }
        )

        PeriodicTask.objects.update_or_create(
            task='corptools.tasks.update_all_corps',
            defaults={
                'crontab': schedule_corp,
                'name': 'Corporation Audit Update',
                'enabled': True
            }
        )

        self.stdout.write("Populating DB models!")
        if options['inline']:
            self.stdout.write("Running Tasks inline this may take some time!")
            self.stdout.write("Starting Update")
            update_models_from_sde()
            self.stdout.write("Done Tasks!")
        else:
            self.stdout.write("Sending Tasks to celery for processing!")
            self.stdout.write("Sending Task")
            update_models_from_sde.apply_async()
            self.stdout.write("Tasks Queued!")



import json

from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


def valid_json(value):
    try:
        json.loads(value)
    except Exception:
        raise ValidationError(
            _('This is not valid JSON, please check it and try again.'))



# Generated by Django 4.0.2 on 2022-08-20 09:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0075_remove_notification_corptools_n_notific_77c7f2_idx'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='characterasset',
            name='corptools_c_type_id_05ccc2_idx',
        ),
        migrations.RemoveIndex(
            model_name='corpasset',
            name='corptools_c_type_id_e9d79e_idx',
        ),
    ]



import datetime
import logging
from typing import ClassVar

from solo.models import SingletonModel

from django.db import models
from django.db.models import ExpressionWrapper, F, Func
from django.utils import timezone

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo

from .. import app_settings
from ..managers import AuditCharacterManager, AuditCorporationManager
from .eve_models import EveItemType, MapSystem

logger = logging.getLogger(__name__)


class EveLocation(models.Model):
    location_id = models.BigIntegerField(primary_key=True)
    location_name = models.CharField(max_length=255)
    system = models.ForeignKey(
        MapSystem, on_delete=models.SET_NULL, null=True, default=None)
    last_update = models.DateTimeField(auto_now=True)
    managed = models.BooleanField(default=False)
    managed_corp = models.ForeignKey(
        "CorporationAudit", default=None, blank=True, null=True, on_delete=models.CASCADE)
    managed_char = models.ForeignKey(
        "CharacterAudit", default=None, blank=True, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return f"{self.location_name}"


class CorptoolsConfiguration(SingletonModel):
    holding_corps = models.ManyToManyField(EveCorporationInfo, blank=True)

    aggregate_lookback = models.IntegerField(
        default=90,
        help_text="Days to look back in aggregated views, like the mining ledger graphs."
    )

    disable_verification_assets = models.BooleanField(
        default=False,
        help_text="Allow ESI to provide data that does not match the ESI Assets Spec"
    )

    disable_verification_notifications = models.BooleanField(
        default=False,
        help_text="Allow ESI to provide data that does not match the ESI Notification Spec"
    )

    disable_update_pub_data = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Pulic History Data"
    )

    disable_update_skills = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character SKill Data"
    )

    disable_update_clones = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Clone Data"
    )

    disable_update_contacts = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Contact Data"
    )

    disable_update_contracts = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Contracts Data"
    )
    disable_update_assets = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Asset Data"
    )

    disable_update_wallet = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Wallet Data"
    )

    disable_update_notif = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Notification Data"
    )

    disable_update_roles = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Roles Data"
    )

    disable_update_mails = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Mail Data"
    )

    disable_update_location = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Location Data"
    )

    disable_update_loyaltypoints = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Loyalty Point Data"
    )

    disable_update_mining = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character Mining Data"
    )

    disable_update_indy = models.BooleanField(
        default=False,
        blank=True,
        help_text="Temporarily disable the ESI pulls for Character industry Job Data"
    )

    class Meta:
        permissions = (
            ('holding_corp_structures',
             'Can access configured holding corp structure data.'),
            ('holding_corp_wallets', 'Can access configured holding corp wallet data.'),
            ('holding_corp_assets', 'Can access configured holding corp asset data.')
        )

        default_permissions = []

    def __str__(self) -> str:
        return "Corptools Configuration"

    def holding_corp_qs(self):
        return CorporationAudit.objects.filter(corporation__in=self.holding_corps.all())

    @classmethod
    def skip_verify_assets(cls):
        try:
            return cls.get_solo().disable_verification_assets
        except Exception as e:
            logger.error(e)
            return True

    @classmethod
    def skip_verify_notifications(cls):
        try:

            return cls.get_solo().disable_verification_notifications
        except Exception as e:
            logger.error(e)
            return True


def check_date(last_update, time_ref):
    date = last_update if last_update is not None else timezone.now() - \
        datetime.timedelta(days=99)
    return date > time_ref


class CharacterAudit(models.Model):

    objects: ClassVar[AuditCharacterManager] = AuditCharacterManager()

    active = models.BooleanField(default=False)

    character = models.OneToOneField(EveCharacter, on_delete=models.CASCADE)

    last_update_pub_data = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_skills = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_skill_que = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_clones = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_contacts = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_contracts = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_assets = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_wallet = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_orders = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_notif = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_roles = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_titles = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_mails = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_location = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_loyaltypoints = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_mining = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_indy = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_login = models.DateTimeField(
        null=True, default=None, blank=True)

    balance = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)

    last_known_login = models.DateTimeField(
        null=True, default=None, blank=True)

    last_known_logoff = models.DateTimeField(
        null=True, default=None, blank=True)

    total_logins = models.IntegerField(
        null=True, default=None, blank=True)

    def __str__(self):
        return f"{self.character.character_name}'s Character Data"

    class Meta:
        permissions = (('corp_hr', 'Can access other character\'s data for own corp.'),
                       ('alliance_hr',
                        'Can access other character\'s data for own alliance.'),
                       ('state_hr', 'Can access other character\'s data for own state.'),
                       ('guest_hr', 'Can access guest users\'s data for HR purposes.'),
                       ('global_hr', 'Can access other character\'s data for characters in any corp/alliance/state.'))

    def is_active(self):
        time_ref = timezone.now() - datetime.timedelta(days=app_settings.CT_CHAR_MAX_INACTIVE_DAYS)
        ct_conf = CorptoolsConfiguration.get_solo()

        try:
            is_active = True
            if app_settings.CT_CHAR_ACTIVE_IGNORE_CORP_HISTORY and not ct_conf.disable_update_pub_data:
                is_active = is_active and check_date(
                    self.last_update_pub_data, time_ref)
            if app_settings.CT_CHAR_ASSETS_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_ASSETS_MODULE and not ct_conf.disable_update_assets:
                is_active = is_active and check_date(
                    self.last_update_assets, time_ref)
            if app_settings.CT_CHAR_CLONES_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_CLONES_MODULE and not ct_conf.disable_update_clones:
                is_active = is_active and check_date(
                    self.last_update_clones, time_ref)
            if app_settings.CT_CHAR_SKILLS_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_SKILLS_MODULE and not ct_conf.disable_update_skills:
                is_active = is_active and check_date(
                    self.last_update_skills, time_ref)
                is_active = is_active and check_date(
                    self.last_update_skill_que, time_ref)
            if app_settings.CT_CHAR_WALLET_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_WALLET_MODULE and not ct_conf.disable_update_wallet:
                is_active = is_active and check_date(
                    self.last_update_wallet, time_ref)
                is_active = is_active and check_date(
                    self.last_update_orders, time_ref)
            if app_settings.CT_CHAR_NOTIFICATIONS_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_NOTIFICATIONS_MODULE and not ct_conf.disable_update_notif:
                is_active = is_active and check_date(
                    self.last_update_notif, time_ref)
            if app_settings.CT_CHAR_ROLES_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_ROLES_MODULE and not ct_conf.disable_update_roles:
                is_active = is_active and check_date(
                    self.last_update_roles, time_ref)
            if app_settings.CT_CHAR_MAIL_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_MAIL_MODULE and not ct_conf.disable_update_mails:
                is_active = is_active and check_date(
                    self.last_update_mails, time_ref)
            if app_settings.CT_CHAR_LOYALTYPOINTS_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_LOYALTYPOINTS_MODULE and not ct_conf.disable_update_loyaltypoints:
                is_active = is_active and check_date(
                    self.last_update_loyaltypoints, time_ref)
            if app_settings.CT_CHAR_MINING_MODULE and not app_settings.CT_CHAR_ACTIVE_IGNORE_MINING_MODULE and not ct_conf.disable_update_mining:
                is_active = is_active and check_date(
                    self.last_update_mining, time_ref)

            if self.active != is_active:
                self.active = is_active
                self.save()

            return is_active
        except Exception as e:
            logger.error(e, exc_info=True)
            return False

    @classmethod
    def get_oldest_qs(cls):
        time_ref = timezone.now() - datetime.timedelta(
            days=app_settings.CT_CHAR_MAX_INACTIVE_DAYS * 3
        )

        ct_conf = CorptoolsConfiguration.get_solo()
        qs = []

        if app_settings.CT_CHAR_ASSETS_MODULE and not (
            app_settings.CT_CHAR_ACTIVE_IGNORE_ASSETS_MODULE or ct_conf.disable_update_assets
        ):
            qs.append(Func(F('last_update_assets'), function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_CLONES_MODULE and not (
            app_settings.CT_CHAR_ACTIVE_IGNORE_CLONES_MODULE or ct_conf.disable_update_clones
        ):
            qs.append(Func(F('last_update_clones'), function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_SKILLS_MODULE and not (
           app_settings.CT_CHAR_ACTIVE_IGNORE_SKILLS_MODULE or ct_conf.disable_update_skills
        ):  # NOQA E124
            qs.append(Func(F('last_update_skills'), function='UNIX_TIMESTAMP'))
            qs.append(Func(F('last_update_skill_que'),
                      function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_WALLET_MODULE and not (
           app_settings.CT_CHAR_ACTIVE_IGNORE_WALLET_MODULE or ct_conf.disable_update_wallet
        ):  # NOQA E124
            qs.append(Func(F('last_update_wallet'), function='UNIX_TIMESTAMP'))
            qs.append(Func(F('last_update_orders'), function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_NOTIFICATIONS_MODULE and not (
           app_settings.CT_CHAR_ACTIVE_IGNORE_NOTIFICATIONS_MODULE or ct_conf.disable_update_notif
        ):  # NOQA E124
            qs.append(Func(F('last_update_notif'), function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_ROLES_MODULE and not (
            app_settings.CT_CHAR_ACTIVE_IGNORE_ROLES_MODULE or ct_conf.disable_update_roles
        ):
            qs.append(Func(F('last_update_roles'), function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_MAIL_MODULE and not (
            app_settings.CT_CHAR_ACTIVE_IGNORE_MAIL_MODULE or ct_conf.disable_update_mails
        ):
            qs.append(Func(F('last_update_mails'), function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_LOYALTYPOINTS_MODULE and not (
            app_settings.CT_CHAR_ACTIVE_IGNORE_LOYALTYPOINTS_MODULE or ct_conf.disable_update_loyaltypoints
        ):
            qs.append(Func(F('last_update_loyaltypoints'),
                      function='UNIX_TIMESTAMP'))

        if app_settings.CT_CHAR_MINING_MODULE and not (
            app_settings.CT_CHAR_ACTIVE_IGNORE_MINING_MODULE or ct_conf.disable_update_mining
        ):
            qs.append(Func(F('last_update_mining'), function='UNIX_TIMESTAMP'))

        tot = len(qs)

        qout = qs.pop()

        for q in qs:
            qout = qout + q

        query = cls.objects.annotate(
            avg_date=Func(
                ExpressionWrapper(
                    qout / tot,
                    output_field=models.BigIntegerField()
                ),
                function='FROM_UNIXTIME',
                output_field=models.DateTimeField()
            )
        ).filter(
            character__character_ownership__isnull=False,
            avg_date__gte=time_ref
        ).order_by("avg_date")

        return query


class CorporationAudit(models.Model):

    objects: ClassVar[AuditCorporationManager] = AuditCorporationManager()

    corporation = models.OneToOneField(
        EveCorporationInfo, on_delete=models.CASCADE)

    last_update_pub_data = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_pub_data = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_assets = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_assets = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_structures = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_structures = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_moons = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_moons = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_observers = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_observers = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_wallet = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_wallet = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_contracts = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_contracts = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_industry_jobs = models.DateTimeField(
        null=True, default=None, blank=True)
    last_change_industry_jobs = models.DateTimeField(
        null=True, default=None, blank=True)

    last_update_known_login = models.DateTimeField(
        null=True, default=None, blank=True)

    def __str__(self):
        return f"{self.corporation.corporation_name}'s Corporation Data"

    class Meta:
        permissions = (
            ('own_corp_manager',
             'Can access own corporations\'s data.'),
            ('alliance_corp_manager',
             'Can access other corporations\'s data for own alliance.'),
            ('state_corp_manager',
             'Can access other corporations\'s data for own state.'),
            ('global_corp_manager', 'Can access all corporations\'s data.'),
            ('show_if_director',
             'Can access all corporations\'s where character is a director.'),
        )


class CharacterLocation(models.Model):
    character = models.OneToOneField(
        CharacterAudit, on_delete=models.CASCADE, related_name="location")

    current_location = models.ForeignKey(
        EveLocation, on_delete=models.CASCADE, null=True, blank=True, default=None)
    current_ship = models.ForeignKey(
        EveItemType, on_delete=models.CASCADE, null=True, blank=True, default=None)
    current_ship_unique = models.BigIntegerField(
        null=True, blank=True, default=None)
    current_ship_name = models.TextField(
        max_length=150, null=True, blank=True, default=None)



# Generated by Django 4.2.16 on 2025-02-23 22:13

from django.db import migrations, models

import corptools.models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0113_timeincorpfilter_reversed_logic'),
    ]

    operations = [
        migrations.AlterField(
            model_name='skilllist',
            name='skill_list',
            field=models.TextField(default='', null=True, validators=[
                                   corptools.models.valid_skills]),
        ),
    ]




def to_roman_numeral(num):
    lookup = [
        (50, 'L'),
        (40, 'XL'),
        (10, 'X'),
        (9, 'IX'),
        (5, 'V'),
        (4, 'IV'),
        (1, 'I'),
    ]
    res = ''
    for (n, roman) in lookup:
        (d, num) = divmod(num, n)
        res += roman * d
    return res



from .assets import *
from .audits import *
from .clones import *
from .contracts import *
from .eve_models import *
from .filters import *
from .Industry import *
from .interactions import *
from .mining import *
from .orders import *
from .skills import *
from .structures import *
from .wallets import *



# Generated by Django 2.2.9 on 2020-05-07 10:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0010_alliance_ticker'),
    ]

    operations = [
        migrations.CreateModel(
            name='CharacterAudit',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('last_update_pub_data', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_pub_data', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_skills', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_skills', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_clones', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_clones', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_assets', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_assets', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('character', models.OneToOneField(
                    on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveCharacter')),
            ],
            options={
                'permissions': (('corp_hr', "Can access other character's data for own corp."), ('alliance_hr', "Can access other character's data for own alliance."), ('state_hr', "Can access other character's data for own state."), ('global_hr', "Can access other character's data for characters in any corp/alliance/state.")),
            },
        ),
        migrations.CreateModel(
            name='CorporationAudit',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('last_update_pub_data', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_pub_data', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_assets', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_assets', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_structures', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_structures', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_moons', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_moons', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('last_update_observers', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('cache_expire_observers', models.DateTimeField(
                    blank=True, default=None, null=True)),
                ('corporation', models.OneToOneField(
                    on_delete=django.db.models.deletion.CASCADE, to='eveonline.EveCorporationInfo')),
            ],
            options={
                'permissions': (('alliance_corp_manager', "Can access other corporations's data for own alliance."), ('state_corp_manager', "Can access other corporations's data for own state."), ('global_corp_manager', "Can access all corporations's data.")),
            },
        ),
        migrations.CreateModel(
            name='EveItemGroup',
            fields=[
                ('group_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('group_name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='EveItemType',
            fields=[
                ('type_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('type_name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='MapConstellation',
            fields=[
                ('constellation_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='MapRegion',
            fields=[
                ('region_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('description', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='MapSystem',
            fields=[
                ('system_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('security_status', models.FloatField()),
                ('name', models.CharField(max_length=255)),
                ('x', models.FloatField()),
                ('y', models.FloatField()),
                ('z', models.FloatField()),
                ('security_class', models.CharField(max_length=255)),
                ('star_id', models.IntegerField()),
                ('constellation', models.ForeignKey(default=None, null=True,
                                                    on_delete=django.db.models.deletion.SET_NULL, to='corptools.MapConstellation')),
            ],
        ),
        migrations.CreateModel(
            name='Structure',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('fuel_expires', models.DateTimeField(default=None, null=True)),
                ('next_reinforce_apply', models.DateTimeField(
                    default=None, null=True)),
                ('next_reinforce_hour', models.IntegerField(default=None, null=True)),
                ('next_reinforce_weekday', models.IntegerField(
                    default=None, null=True)),
                ('profile_id', models.IntegerField()),
                ('reinforce_hour', models.IntegerField()),
                ('reinforce_weekday', models.IntegerField(default=None, null=True)),
                ('state', models.CharField(choices=[('anchor_vulnerable', 'anchor_vulnerable'), ('anchoring', 'anchoring'), ('armor_reinforce', 'armor_reinforce'), ('armor_vulnerable', 'armor_vulnerable'), ('deploy_vulnerable', 'deploy_vulnerable'), ('fitting_invulnerable', 'fitting_invulnerable'), (
                    'hull_reinforce', 'hull_reinforce'), ('hull_vulnerable', 'hull_vulnerable'), ('online_deprecated', 'online_deprecated'), ('onlining_vulnerable', 'onlining_vulnerable'), ('shield_vulnerable', 'shield_vulnerable'), ('unanchored', 'unanchored'), ('unknown', 'unknown')], max_length=25)),
                ('state_timer_end', models.DateTimeField(default=None, null=True)),
                ('state_timer_start', models.DateTimeField(default=None, null=True)),
                ('structure_id', models.BigIntegerField()),
                ('system_id', models.IntegerField()),
                ('type_id', models.IntegerField()),
                ('unanchors_at', models.DateTimeField(default=None, null=True)),
                ('name', models.CharField(max_length=150)),
            ],
        ),
        migrations.CreateModel(
            name='StructureCelestial',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('structure_id', models.BigIntegerField()),
                ('celestial_name', models.CharField(
                    default=None, max_length=500, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='StructureService',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('state', models.CharField(choices=[
                 ('online', 'online'), ('offline', 'offline'), ('cleanup', 'cleanup')], max_length=8)),
                ('structure', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.Structure')),
            ],
        ),
        migrations.AddField(
            model_name='structure',
            name='closest_celestial',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.StructureCelestial'),
        ),
        migrations.AddField(
            model_name='structure',
            name='corporation',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='ct_structure', to='corptools.CorporationAudit'),
        ),
        migrations.AddField(
            model_name='structure',
            name='system_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.MapSystem'),
        ),
        migrations.AddField(
            model_name='structure',
            name='type_name',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveItemType'),
        ),
        migrations.CreateModel(
            name='SkillQueue',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('finish_level', models.IntegerField()),
                ('queue_position', models.IntegerField()),
                ('skill_id', models.IntegerField()),
                ('finish_date', models.DateTimeField(default=None, null=True)),
                ('level_end_sp', models.IntegerField(default=None, null=True)),
                ('level_start_sp', models.IntegerField(default=None, null=True)),
                ('start_date', models.DateTimeField(default=None, null=True)),
                ('training_start_sp', models.IntegerField(default=None, null=True)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
                ('skill_name', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.EveItemType')),
            ],
        ),
        migrations.CreateModel(
            name='Skill',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('skill_id', models.IntegerField()),
                ('active_skill_level', models.IntegerField()),
                ('skillpoints_in_skill', models.BigIntegerField()),
                ('trained_skill_level', models.IntegerField()),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
            ],
        ),
        migrations.AddField(
            model_name='mapconstellation',
            name='region',
            field=models.ForeignKey(
                default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='corptools.MapRegion'),
        ),
        migrations.CreateModel(
            name='EveName',
            fields=[
                ('eve_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('category', models.CharField(max_length=50)),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('alliance', models.ForeignKey(default=None, null=True,
                                               on_delete=django.db.models.deletion.SET_NULL, related_name='alli', to='corptools.EveName')),
                ('corporation', models.ForeignKey(default=None, null=True,
                                                  on_delete=django.db.models.deletion.SET_NULL, related_name='corp', to='corptools.EveName')),
            ],
        ),
        migrations.CreateModel(
            name='CorporationHistory',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('corporation_id', models.IntegerField()),
                ('is_deleted', models.NullBooleanField(default=None)),
                ('record_id', models.IntegerField()),
                ('start_date', models.DateTimeField()),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
                ('corporation_name', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.EveName')),
            ],
        ),
        migrations.CreateModel(
            name='CorpAsset',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('blueprint_copy', models.NullBooleanField(default=None)),
                ('singleton', models.BooleanField()),
                ('item_id', models.BigIntegerField(unique=True)),
                ('location_flag', models.CharField(max_length=50)),
                ('location_id', models.BigIntegerField()),
                ('location_type', models.CharField(max_length=25)),
                ('quantity', models.IntegerField()),
                ('type_id', models.IntegerField()),
                ('name', models.CharField(default=None, max_length=255, null=True)),
                ('corporation', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CorporationAudit')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CharacterAsset',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('blueprint_copy', models.NullBooleanField(default=None)),
                ('singleton', models.BooleanField()),
                ('item_id', models.BigIntegerField(unique=True)),
                ('location_flag', models.CharField(max_length=50)),
                ('location_id', models.BigIntegerField()),
                ('location_type', models.CharField(max_length=25)),
                ('quantity', models.IntegerField()),
                ('type_id', models.IntegerField()),
                ('name', models.CharField(default=None, max_length=255, null=True)),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import datetime

from model_utils import Choices

from django.db import models
from django.utils import timezone

from .audits import CorporationAudit, CorptoolsConfiguration
from .eve_models import EveItemType, MapSystem, MapSystemMoon, MapSystemPlanet


class BridgeOzoneLevel(models.Model):
    id = models.AutoField(primary_key=True)
    station_id = models.CharField(max_length=500)
    quantity = models.BigIntegerField()
    used = models.BigIntegerField(default=0)
    date = models.DateTimeField(auto_now=True)


class StructureCelestial(models.Model):
    structure_id = models.BigIntegerField()
    celestial_name = models.CharField(max_length=500, null=True, default=None)


class Structure(models.Model):
    corporation = models.ForeignKey(
        CorporationAudit, on_delete=models.CASCADE, related_name='ct_structure')

    fuel_expires = models.DateTimeField(null=True, default=None)
    next_reinforce_apply = models.DateTimeField(null=True, default=None)
    next_reinforce_hour = models.IntegerField(null=True, default=None)
    next_reinforce_weekday = models.IntegerField(null=True, default=None)
    profile_id = models.IntegerField()
    reinforce_hour = models.IntegerField()
    reinforce_weekday = models.IntegerField(null=True, default=None)
    _state_enum = Choices('anchor_vulnerable', 'anchoring', 'armor_reinforce', 'armor_vulnerable', 'deploy_vulnerable',
                          'fitting_invulnerable', 'hull_reinforce', 'hull_vulnerable', 'online_deprecated',
                          'onlining_vulnerable', 'shield_vulnerable', 'unanchored', 'unknown')
    state = models.CharField(max_length=25, choices=_state_enum)
    state_timer_end = models.DateTimeField(null=True, default=None)
    state_timer_start = models.DateTimeField(null=True, default=None)
    structure_id = models.BigIntegerField()
    system_id = models.IntegerField()
    type_id = models.IntegerField()
    unanchors_at = models.DateTimeField(null=True, default=None)

    # extra
    name = models.CharField(max_length=150)
    system_name = models.ForeignKey(
        MapSystem, on_delete=models.SET_NULL, null=True, default=None)
    type_name = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)
    closest_celestial = models.ForeignKey(
        StructureCelestial, on_delete=models.SET_NULL, null=True, default=None)
    last_online_time = models.DateTimeField(null=True, default=None)

    @property
    def services(self):
        return StructureService.objects.filter(structure=self)

    @property
    def ozone_level(self):
        try:
            last_ozone = BridgeOzoneLevel.objects.filter(
                station_id=self.structure_id).order_by('-date')[:1][0].quantity
            return last_ozone
        except Exception:
            return False

    @property
    def abandoned(self):
        if self.last_online_time:
            abandonded = self.last_online_time + datetime.timedelta(days=7)
            if timezone.now() > abandonded:
                return True
            else:
                return False
        return True  # Fallback to abandonded. wosrt case its 7 days early

    @classmethod
    def get_visible(cls, user):
        corps_vis = CorporationAudit.objects.visible_to(user)
        if user.has_perm("corptools.holding_corp_structures"):
            corps_holding = CorptoolsConfiguration.get_solo().holding_corp_qs()
            corps_vis = corps_vis | corps_holding
        update_time_filter = timezone.now() - datetime.timedelta(days=7)
        return cls.objects.filter(corporation__in=corps_vis, corporation__last_update_structures__gte=update_time_filter)


class Poco(models.Model):
    corporation = models.ForeignKey(
        CorporationAudit, on_delete=models.CASCADE, related_name='ct_poco')

    office_id = models.BigIntegerField()

    standing_level = models.CharField(max_length=50)

    alliance_tax_rate = models.FloatField(null=True, default=None)
    corporation_tax_rate = models.FloatField(null=True, default=None)

    terrible_standing_tax_rate = models.FloatField(null=True, default=None)
    bad_standing_tax_rate = models.FloatField(null=True, default=None)
    neutral_standing_tax_rate = models.FloatField(null=True, default=None)
    good_standing_tax_rate = models.FloatField(null=True, default=None)
    excellent_standing_tax_rate = models.FloatField(null=True, default=None)

    reinforce_exit_end = models.IntegerField()
    reinforce_exit_start = models.IntegerField()

    standing_level = models.CharField(max_length=150, null=True, default=None)

    allow_access_with_standings = models.BooleanField(null=True, default=None)
    allow_alliance_access = models.BooleanField(null=True, default=None)

    system_id = models.IntegerField()

    name = models.CharField(max_length=150, null=True,
                            default=None, blank=True)

    planet = models.ForeignKey(
        MapSystemPlanet, on_delete=models.SET_NULL, null=True, default=None)
    system_name = models.ForeignKey(
        MapSystem, on_delete=models.SET_NULL, null=True, default=None)

    @classmethod
    def get_visible(cls, user):
        corps_vis = CorporationAudit.objects.visible_to(user)
        if user.has_perm("corptools.holding_corp_structures"):
            corps_holding = CorptoolsConfiguration.get_solo().holding_corp_qs()
            corps_vis = corps_vis | corps_holding
        # update_time_filter = timezone.now() - datetime.timedelta(days=7)
        # , corporation__last_update_pocos__gte=update_time_filter)
        return cls.objects.filter(corporation__in=corps_vis)


class StructureService(models.Model):
    structure = models.ForeignKey(Structure, on_delete=models.CASCADE)

    name = models.CharField(max_length=100)
    _state_enum = Choices('online', 'offline', 'cleanup')
    state = models.CharField(max_length=8, choices=_state_enum)


class Starbase(models.Model):
    corporation = models.ForeignKey(
        CorporationAudit,
        on_delete=models.CASCADE
    )
    starbase_id = models.BigIntegerField()
    name = models.CharField(max_length=255, default="", null=True, blank=True)
    onlined_since = models.DateTimeField(null=True, default=None)
    reinforced_until = models.DateTimeField(null=True, default=None)
    unanchor_at = models.DateTimeField(null=True, default=None)

    moon = models.ForeignKey(
        MapSystemMoon,
        on_delete=models.CASCADE,
        null=True,
        default=None
    )
    system = models.ForeignKey(
        MapSystem,
        on_delete=models.CASCADE,
        null=True,
        default=None
    )
    type_name = models.ForeignKey(
        EveItemType,
        on_delete=models.CASCADE,
        null=True,
        default=None
    )

    allow_alliance_members = models.BooleanField(default=False)
    allow_corporation_members = models.BooleanField(default=False)
    attack_if_at_war = models.BooleanField(default=False)
    attack_if_other_security_status_dropping = models.BooleanField(
        default=False)
    use_alliance_standings = models.BooleanField(default=False)

    attack_security_status_threshold = models.FloatField(
        null=True, default=None)
    attack_standing_threshold = models.FloatField(null=True, default=None)

    _state_enum = Choices('online', 'offline', 'onlining',
                          'reinforced', 'unanchoring')
    state = models.CharField(max_length=12, choices=_state_enum)

    _perm_enum = Choices(
        'alliance_member',
        'config_starbase_equipment_role',
        'corporation_member',
        'starbase_fuel_technician_role'
    )

    anchor = models.CharField(
        max_length=31,
        choices=_perm_enum,
        null=True,
        default=None
    )
    online = models.CharField(
        max_length=31,
        choices=_perm_enum,
        null=True,
        default=None
    )
    offline = models.CharField(
        max_length=31,
        choices=_perm_enum,
        null=True,
        default=None
    )
    unanchor = models.CharField(
        max_length=31,
        choices=_perm_enum,
        null=True,
        default=None
    )
    fuel_bay_take = models.CharField(
        max_length=31,
        choices=_perm_enum,
        null=True,
        default=None
    )
    fuel_bay_view = models.CharField(
        max_length=31,
        choices=_perm_enum,
        null=True,
        default=None
    )

    fuels = models.TextField(null=True, default="", blank=True)

    @classmethod
    def get_visible(cls, user):
        corps_vis = CorporationAudit.objects.visible_to(user)
        if user.has_perm("corptools.holding_corp_structures"):
            corps_holding = CorptoolsConfiguration.get_solo().holding_corp_qs()
            corps_vis = corps_vis | corps_holding
        # update_time_filter = timezone.now() - datetime.timedelta(days=7)
        return cls.objects.filter(corporation__in=corps_vis)



# Generated by Django 2.2.12 on 2020-08-26 07:55

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0029_auto_20200815_0554'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorporationMarketOrder',
            fields=[
                ('order_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('duration', models.IntegerField()),
                ('escrow', models.IntegerField(default=None, null=True)),
                ('is_buy_order', models.NullBooleanField(default=None)),
                ('issued', models.DateTimeField()),
                ('location_id', models.BigIntegerField()),
                ('min_volume', models.IntegerField(default=None, null=True)),
                ('price', models.DecimalField(decimal_places=2,
                                              default=None, max_digits=20, null=True)),
                ('order_range', models.CharField(choices=[('1', '1'), ('10', '10'), ('2', '2'), ('20', '20'), ('3', '3'), ('30', '30'), ('4', '4'), (
                    '40', '40'), ('5', '5'), ('region', 'region'), ('solarsystem', 'solarsystem'), ('station', 'station')], max_length=30)),
                ('region_id', models.IntegerField()),
                ('type_id', models.IntegerField()),
                ('volume_remain', models.IntegerField()),
                ('volume_total', models.IntegerField()),
                ('state', models.CharField(choices=[
                 ('cancelled', 'cancelled'), ('expired', 'expired'), ('active', 'active')], max_length=30)),
                ('issued_by', models.IntegerField()),
                ('location_name', models.ForeignKey(default=None, null=True,
                                                    on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveLocation')),
                ('wallet_division', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CorporationWalletDivision')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CharacterMarketOrder',
            fields=[
                ('order_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('duration', models.IntegerField()),
                ('escrow', models.IntegerField(default=None, null=True)),
                ('is_buy_order', models.NullBooleanField(default=None)),
                ('issued', models.DateTimeField()),
                ('location_id', models.BigIntegerField()),
                ('min_volume', models.IntegerField(default=None, null=True)),
                ('price', models.DecimalField(decimal_places=2,
                                              default=None, max_digits=20, null=True)),
                ('order_range', models.CharField(choices=[('1', '1'), ('10', '10'), ('2', '2'), ('20', '20'), ('3', '3'), ('30', '30'), ('4', '4'), (
                    '40', '40'), ('5', '5'), ('region', 'region'), ('solarsystem', 'solarsystem'), ('station', 'station')], max_length=30)),
                ('region_id', models.IntegerField()),
                ('type_id', models.IntegerField()),
                ('volume_remain', models.IntegerField()),
                ('volume_total', models.IntegerField()),
                ('state', models.CharField(choices=[
                 ('cancelled', 'cancelled'), ('expired', 'expired'), ('active', 'active')], max_length=30)),
                ('is_corporation', models.BooleanField()),
                ('character', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.CharacterAudit')),
                ('location_name', models.ForeignKey(default=None, null=True,
                                                    on_delete=django.db.models.deletion.SET_NULL, to='corptools.EveLocation')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
